<?php

namespace Ems\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO;

use Ems\Services\IndicadoresSegurancaTrabalhoService;

class IndicadoresSegurancaTrabalhoController {
    /** @var IndicadoresSegurancaTrabalhoService */
    protected $indicadoresSegurancaTrabalhoService;

    public function __construct(
        IndicadoresSegurancaTrabalhoService $indicadoresSegurancaTrabalhoService
    ) {
        $this->indicadoresSegurancaTrabalhoService = $indicadoresSegurancaTrabalhoService;
    }

    public function getTotalAcidentes(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();

            $totais = $this->indicadoresSegurancaTrabalhoService->getTotalAcidentes($row);

            $response->addDataSet(new DataSet('data', $totais));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    
    public function getTotalAcidentesGerenOcup(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();
            
            $totais = $this->indicadoresSegurancaTrabalhoService->getTotalAcidentesGerenOcup($row);

            $response->addDataSet(new DataSet('data', $totais));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTotalAcidentesOcupacaoGeral(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();

            $totais = $this->indicadoresSegurancaTrabalhoService->getTotalAcidentesOcupacaoGeral($row);

            $response->addDataSet(new DataSet('data', $totais));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTotalAcidentesPorGravidadeLegalOcupacao(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();

            $totais = $this->indicadoresSegurancaTrabalhoService->getTotalAcidentesPorGravidadeLegalOcupacao($row);

            $response->addDataSet(new DataSet('data', $totais));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTotalAcidentesPorGravidadeGerenOcup(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();

            $totais = $this->indicadoresSegurancaTrabalhoService->getTotalAcidentesPorGravidadeGerenOcup($row);

            $response->addDataSet(new DataSet('data', $totais));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTotalAcidentesPorGravidadeOcupacaoGeral(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();

            $totais = $this->indicadoresSegurancaTrabalhoService->getTotalAcidentesPorGravidadeOcupacaoGeral($row);

            $response->addDataSet(new DataSet('data', $totais));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getQuantidadeDiasAfastadosLegalOcup(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();

            $totais = $this->indicadoresSegurancaTrabalhoService->getQuantidadeDiasAfastadosLegalOcup($row);

            $response->addDataSet(new DataSet('data', $totais));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getQuantidadeDiasAfastadosOcupacaoGeral(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();

            $totais = $this->indicadoresSegurancaTrabalhoService->getQuantidadeDiasAfastadosOcupacaoGeral($row);

            $response->addDataSet(new DataSet('data', $totais));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTotalPartesDoCorpoAtingidasLegalOcup(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();

            $totais = $this->indicadoresSegurancaTrabalhoService->getTotalPartesDoCorpoAtingidasLegalOcup($row);

            $response->addDataSet(new DataSet('data', $totais));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTotalPartesDoCorpoAtingidasOcupacaoGeral(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();

            $totais = $this->indicadoresSegurancaTrabalhoService->getTotalPartesDoCorpoAtingidasOcupacaoGeral($row);

            $response->addDataSet(new DataSet('data', $totais));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getSituacaoGeradoraAcidenteLegalOcup(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();

            $totais = $this->indicadoresSegurancaTrabalhoService->getSituacaoGeradoraAcidenteLegalOcup($row);

            $response->addDataSet(new DataSet('data', $totais));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getSituacaoGeradoraAcidenteOcupacaoGeral(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();

            $totais = $this->indicadoresSegurancaTrabalhoService->getSituacaoGeradoraAcidenteOcupacaoGeral($row);

            $response->addDataSet(new DataSet('data', $totais));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }


    
}