<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Util\{ DateUtil };

class BeneficioRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    /**
     * @return array|false
     */
    public function getBeneficio(
        int $nrOrg,
        int $nrBeneficio
    ) {
        $dtAtual = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);

        $sql = <<<SQL
            SELECT
                BM.*,
                BH.NREVENTOMENSADEP
            FROM
                VAB_BENEFICIOM BM
                JOIN
                VAB_BENEFICIOH BH ON
                    BH.NRBENEFICIOM = BM.NRBENEFICIOM AND
                    BH.NRORG = BM.NRORG AND
                    BH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            VAB_BENEFICIOH BH1
                        WHERE
                            BH1.NRORG = BM.NRORG AND
                            BH1.NRBENEFICIOM = BM.NRBENEFICIOM
                    )
            WHERE
                BM.NRORG = :P_NRORG AND
                BM.NRBENEFICIOM = :P_NRBENEFICIOM AND
                (BM.DTFINALVIGENCIA >= :P_DTATUAL OR BM.DTFINALVIGENCIA IS NULL) AND
                BM.IDATIVO = 'S'
SQL;

        $params = [
            'P_NRORG'        => $nrOrg,
            'P_NRBENEFICIOM' => $nrBeneficio,
            'P_DTATUAL'      => $dtAtual
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetch();

        return $result;
    }

    public function getBeneficios(
        int $nrOrg
    ): array {
        $dtAtual = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);

        $sql = <<<SQL
            SELECT
                *
            FROM
                VAB_BENEFICIOM BM
                JOIN
                VAB_BENEFICIOH BH ON
                    BH.NRBENEFICIOM = BM.NRBENEFICIOM AND
                    BH.NRORG = BM.NRORG AND
                    BH.DTMESCOMPETENC = (
                        (SELECT DTMESCOMPETENC FROM 
                            (SELECT 
                                BH1.DTMESCOMPETENC 
                            FROM 
                                VAB_BENEFICIOH BH1 
                            WHERE 
                                BH1.NRORG = BM.NRORG AND
                                BH1.NRBENEFICIOM = BM.NRBENEFICIOM
                            ORDER BY BH1.DTMESCOMPETENC DESC) 
                        WHERE ROWNUM = 1)
                    )
            WHERE
                BM.NRORG = :P_NRORG AND
                (BM.DTFINALVIGENCIA >= :P_DTATUAL OR BM.DTFINALVIGENCIA IS NULL) AND
                BM.IDATIVO = 'S'
SQL;

        $params = [
            'P_NRORG'   => $nrOrg,
            'P_DTATUAL' => $dtAtual
        ];
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        $beneficios = $std->fetchAll();
        
        return $beneficios;
    }

    public function getBeneficiosComFiltro(
        int $nrOrg
    ): array {
        $dtAtual = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);

        $sql = <<<SQL
            SELECT
                BM.NRBENEFICIOM,
                BM.NRBENEFICIOM || ' | ' || BH.DSBENEFICIO AS NRDSBENEFICIO,
                BH.NRBENEFICIOH,
                BH.DSBENEFICIO,
                BH.TPPERIODOBENEF,
                BH.NRTPBENEFICIO
            FROM
                VAB_BENEFICIOM BM
                JOIN VAB_BENEFICIOH BH ON
                    BH.NRBENEFICIOM = BM.NRBENEFICIOM AND
                    BH.NRORG = BM.NRORG AND
                    BH.DTMESCOMPETENC = (
                        (SELECT DTMESCOMPETENC FROM 
                            (SELECT 
                                BH1.DTMESCOMPETENC 
                            FROM 
                                VAB_BENEFICIOH BH1 
                            WHERE 
                                BH1.NRORG = BM.NRORG AND
                                BH1.NRBENEFICIOM = BM.NRBENEFICIOM
                            ORDER BY BH1.DTMESCOMPETENC DESC) 
                        WHERE ROWNUM = 1)
                    )
                JOIN TPBENEFICIO TB ON
                TB.NRORG = BM.NRORG AND
                TB.NRTPBENEFICIO = BH.NRTPBENEFICIO AND
                NOT (UPPER(TB.DSTPBENEFICIO) LIKE UPPER('%combustível%') OR UPPER(TB.DSTPBENEFICIO) LIKE UPPER('%combustivel%')) AND
                (BM.DTFINALVIGENCIA >= :P_DTATUAL OR BM.DTFINALVIGENCIA IS NULL) AND
                BM.IDATIVO = 'S'
            WHERE
                BM.NRORG = :P_NRORG AND
                (BM.DTFINALVIGENCIA >= :P_DTATUAL OR BM.DTFINALVIGENCIA IS NULL) AND
                BM.IDATIVO = 'S'
SQL;

        $params = [
            'P_NRORG'   => $nrOrg,
            'P_DTATUAL' => $dtAtual
        ];
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        $beneficios = $std->fetchAll();
        
        return $beneficios;
    }


    public function getTipoBeneficio(
        int $nrOrg
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                TPBENEFICIO TB
            WHERE
                TB.NRORG = :P_NRORG AND
                TB.IDATIVO = 'S' 
SQL;

        $params = [
            'P_NRORG' => $nrOrg
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getTipoBeneficioComFiltro(
        int $nrOrg
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                TPBENEFICIO TB
            WHERE
                TB.NRORG = :P_NRORG AND
                NOT (UPPER(TB.DSTPBENEFICIO) LIKE UPPER('%combustível%') OR UPPER(TB.DSTPBENEFICIO) LIKE UPPER('%combustivel%')) AND
                TB.IDATIVO = 'S' 
SQL;

        $params = [
            'P_NRORG' => $nrOrg
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function setBeneficiario(
        array $params
    ): void {
        $sql = <<<SQL
            INSERT INTO VAB_BENEFICIARIO (
                NRORG,                  NRBENEFICIARIO,         NRBENEFICIOM,
                NRVINCULOM,             NRPESSOA,               NREVENTOM,
                NRINTEGTRANSPM,         NRVALORREFERENCIAM,     NRVINCEVENTO,
                DTINSCBENEFICIARIO,     NRINSCBENEFICIARIO,     DTCANCBENEFICIARIO,
                QTBENEFICIO,            VRBENEFICIO,            IDTPBENEFICIARIO,
                IDVALORDIFEBENEF,       DSOBSBENEFICIARIO,      IDATIVO,
                NRORGINCLUSAO,          DTINCLUSAO,             CDOPERINCLUSAO,
                NRORGULTATU,            DTULTATU,               CDOPERULTATU,
                SUBSIDIO100,            VRPERCSUBSIDIO
            )
            VALUES (
                :NRORG,                 :NRBENEFICIARIO,        :NRBENEFICIOM,
                :NRVINCULOM,            :NRPESSOA,              :NREVENTOM,
                :NRINTEGTRANSPM,        :NRVALORREFERENCIAM,    :NRVINCEVENTO,
                :DTINSCBENEFICIARIO,    :NRINSCBENEFICIARIO,    :DTCANCBENEFICIARIO,
                :QTBENEFICIO,           :VRBENEFICIO,           :IDTPBENEFICIARIO,
                :IDVALORDIFEBENEF,      :DSOBSBENEFICIARIO,     :IDATIVO,
                :NRORGINCLUSAO,         :DTINCLUSAO,            :CDOPERINCLUSAO,
                :NRORGULTATU,           :DTULTATU,              :CDOPERULTATU,
                :SUBSIDIO100,           :VRPERCSUBSIDIO
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getBeneficiosVCombust(
        int $nrOrg
    ) {
        $dtAtual = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);

        $sql = <<<SQL
            SELECT
                BM.*,
                BM.NRBENEFICIOM || ' | ' || BH.DSBENEFICIO AS NRDSBENEFICIO,
                BH.NRBENEFICIOH,
                BH.DSBENEFICIO,
                BH.TPPERIODOBENEF,
                BH.NRTPBENEFICIO
            FROM
                VAB_BENEFICIOM BM
                JOIN
                VAB_BENEFICIOH BH ON
                    BH.NRBENEFICIOM = BM.NRBENEFICIOM AND
                    BH.NRORG = BM.NRORG AND
                    BH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            VAB_BENEFICIOH BH1
                        WHERE
                            BH1.NRORG = BM.NRORG AND
                            BH1.NRBENEFICIOM = BM.NRBENEFICIOM
                    )
                JOIN TPBENEFICIO TB ON
                TB.NRORG = BM.NRORG AND
                TB.NRTPBENEFICIO = BH.NRTPBENEFICIO AND
                (UPPER(TB.DSTPBENEFICIO) LIKE UPPER('%combustível%') OR UPPER(TB.DSTPBENEFICIO) LIKE UPPER('%combustivel%')) AND
                (BM.DTFINALVIGENCIA >= :P_DTATUAL OR BM.DTFINALVIGENCIA IS NULL) AND
                BM.IDATIVO = 'S'
            WHERE
                BM.NRORG = :P_NRORG
                AND (BM.DTFINALVIGENCIA >= :P_DTATUAL OR BM.DTFINALVIGENCIA IS NULL)
                AND BM.IDATIVO = 'S'

SQL;

        $params = [
            'P_NRORG'   => $nrOrg,
            'P_DTATUAL' => $dtAtual
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getBeneficiosVManutencao(
        int $nrOrg
    ) {
        $dtAtual = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);

        $sql = <<<SQL
            SELECT
                BM.*,
                BM.NRBENEFICIOM || ' | ' || BH.DSBENEFICIO AS NRDSBENEFICIO,
                BH.NRBENEFICIOH,
                BH.DSBENEFICIO,
                BH.TPPERIODOBENEF,
                BH.NRTPBENEFICIO
            FROM
                VAB_BENEFICIOM BM
                JOIN VAB_BENEFICIOH BH ON
                    BH.NRBENEFICIOM = BM.NRBENEFICIOM AND
                    BH.NRORG = BM.NRORG AND
                    BH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            VAB_BENEFICIOH BH1
                        WHERE
                            BH1.NRORG = BM.NRORG AND
                            BH1.NRBENEFICIOM = BM.NRBENEFICIOM
                    )
                JOIN TPBENEFICIO TB ON
                    TB.NRORG = BM.NRORG AND
                    TB.NRTPBENEFICIO = BH.NRTPBENEFICIO AND
                    NLS_UPPER(NLSSORT(TB.DSTPBENEFICIO, 'NLS_SORT=BINARY_AI')) = NLS_UPPER(NLSSORT('manutenção', 'NLS_SORT=BINARY_AI')) AND
                    (BM.DTFINALVIGENCIA >= :P_DTATUAL OR BM.DTFINALVIGENCIA IS NULL) AND
                    BM.IDATIVO = 'S'
                    
            WHERE
                BM.NRORG = :P_NRORG AND
                (BM.DTFINALVIGENCIA >= :P_DTATUAL OR BM.DTFINALVIGENCIA IS NULL) AND
                BM.IDATIVO = 'S'
SQL;

        $params = [
            'P_NRORG'   => $nrOrg,
            'P_DTATUAL' => $dtAtual
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }
}