<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Ems\Util\Utils;
use Util\NovoCodigo;
use Ems\Repository\PeriodoRepository;
use Ems\Util\Utilities;

class OcorrenciaRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager,
        PeriodoRepository $periodoRepository
    ) {
        $this->connection           = $entityManager->getConnection();
        $this->periodoRepository    = $periodoRepository;
        $this->entityManager        = $entityManager;
    }

    public function getOcorrencia(
        int $nrOrg,
        int $nrOcorrencia
    ): array {
        $params = [
            'P_NRORG'        => $nrOrg,
            'P_NROCORRENCIA' => $nrOcorrencia
        ];

        $sql = <<<SQL
            SELECT *
            FROM PTO_PROGOCORRENCIA
            WHERE NRORG = :P_NRORG
              AND NRPROGOCORRENCIA = :P_NROCORRENCIA
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $ocorrencia = $std->fetch();

        return is_array($ocorrencia) ? $ocorrencia : [];
    }

    public function getTipoProgocorrencia(string $nrorg, string $nrtipoprogocorrencia) : string {
        try {
            $sql = <<<SQL
                SELECT *
                FROM PTO_TIPOPROGOCORRENCIA TP
                WHERE (TP.NRORG = :P_NRORG OR TP.NRORG = :P_NRORG_PADRAO)
                            AND TP.NRTIPOPROGOCORRENCIA = :P_NRTIPOPROGOCORRENCIA
            ORDER BY TP.NMTIPOPROGOCORRENCIA
SQL;
            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"                   => $nrorg,
                    "P_NRTIPOPROGOCORRENCIA"    => $nrtipoprogocorrencia,
                    "P_NRORG_PADRAO"            => 0
                )
            );

            $result = $query->fetchAll();

            $idprogocorconfirmada = '';
            if ($result[0]['IDPROGOCORACONFIRMAR'] == "S") {
                $idprogocorconfirmada = "P";
            } else {
                $idprogocorconfirmada = "S";
            }
            return $idprogocorconfirmada;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     *  Rotina que salva as ocorrências de marcação de ponto
     * 
     * 
     *  Campos do tipo string são NOT_NULL
     */
    public function geraOcorrenciaFrequencia(
        string  $nrvinculom, 
        string  $dtinicioprogocor, 
                $dtfimprogocor, 
                $iddiassemana, 
                $idtipocalendario, 
        string  $idtipohorario, 
                $nrhordiam, 
                $nrintervalohorario, 
                $nrmotivoocorfreq, 
                $qtmaximaprogocor, 
                $dsobservacao, 
                $nrtipoprogocorrencia, 
                $nrmarcacaoponto, 
        string  $idprogocorconfirmada, 
                $dsobservacaorescusa, 
                $hrprojetarintervalonprog, 
                $dtinclusao, 
                $cdoperinclusao, 
        string  $nrorg
    ) : int {
        
        try {
            $nrprogocorrencia = NovoCodigo::geraCodigo($nrorg, 'PTO_PROGOCORRENCIA', 1, 22, false);

            $sql = <<<SQL
                INSERT INTO PTO_PROGOCORRENCIA (
                    NRPROGOCORRENCIA,
                    NRVINCULOM,
                    DTINICIOPROGOCOR,
                    DTFIMPROGOCOR,
                    IDDIASSEMANA,
                    IDTIPODIACALENDARIO,
                    IDTIPOHORARIO,
                    NRHORDIAM,
                    NRINTERVALOHORARIO,
                    NRMOTIVOOCORFREQ,
                    QTMAXIMAPROGOCOR,
                    DSOBSERVACAO,
                    NRTIPOPROGOCORRENCIA,
                    NRMARCACAOPONTO,
                    IDPROGOCORCONFIRMADA,
                    DSOBSERVACAORECUSA,
                    HRPROJETARINTERVALONPROG,
                    DTINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORG
                ) VALUES (
                    :P_NRPROGOCORRENCIA,
                    :P_NRVINCULOM,
                    TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY'),
                    TO_DATE(:P_DTFIMPROGOCOR, 'DD/MM/YYYY'),
                    :P_IDDIASSEMANA,
                    :P_IDTIPOCALENDARIO,
                    :P_IDTIPOHORARIO,
                    :P_NRHORDIAM,
                    :P_NRINTERVALOHORARIO,
                    :P_NRMOTIVOOCORFREQ,
                    :P_QTMAXIMAPROGOCOR,
                    :P_DSOBSERVACAO,
                    :P_NRTIPOPROGOCORRENCIA,
                    :P_NRMARCACAOPONTO,
                    :P_IDPROGOCORCONFIRMADA,
                    :P_DSOBSERVACAORESCUSA,
                    :P_HRPROJETARINTERVALONPROG,
                    TO_DATE(:P_DTINCLUSAO, 'DD/MM/YYYY'),
                    :P_CDOPERINCLUSAO,
                    :P_NRORG
                )
SQL;

            $query = $this->connection->prepare($sql);

            $result = $query->execute(
                array(
                    "P_NRPROGOCORRENCIA"            => $nrprogocorrencia,
                    "P_NRVINCULOM"                  => $nrvinculom,
                    "P_DTINICIOPROGOCOR"            => $dtinicioprogocor,
                    "P_DTFIMPROGOCOR"               => $dtfimprogocor,
                    "P_IDDIASSEMANA"                => $iddiassemana,
                    "P_IDTIPOCALENDARIO"            => $idtipocalendario,
                    "P_IDTIPOHORARIO"               => $idtipohorario,
                    "P_NRHORDIAM"                   => $nrhordiam,
                    "P_NRINTERVALOHORARIO"          => $nrintervalohorario,
                    "P_NRMOTIVOOCORFREQ"            => $nrmotivoocorfreq,
                    "P_QTMAXIMAPROGOCOR"            => $qtmaximaprogocor,
                    "P_DSOBSERVACAO"                => $dsobservacao,
                    "P_NRTIPOPROGOCORRENCIA"        => $nrtipoprogocorrencia,
                    "P_NRMARCACAOPONTO"             => $nrmarcacaoponto,
                    "P_IDPROGOCORCONFIRMADA"        => $idprogocorconfirmada,
                    "P_DSOBSERVACAORESCUSA"         => $dsobservacaorescusa,
                    "P_HRPROJETARINTERVALONPROG"    => $hrprojetarintervalonprog,
                    "P_DTINCLUSAO"                  => $dtinclusao,
                    "P_CDOPERINCLUSAO"              => $cdoperinclusao,
                    "P_NRORG"                       => $nrorg
                )
            );

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function removeOcorrenciaFrequencia(string $nrvinculom, string $dtinicioprogocor, string $dtfimprogocor, string $nrtipoprogocorrencia, string $nrmarcacaoponto, string $nrprogocorrencia, string $nrorg) : bool {
        try {
            if ($nrmarcacaoponto != '') {
                $condition = "AND NRMARCACAOPONTO = $nrmarcacaoponto";
            } elseif ($nrprogocorrencia != '') {
                $condition = "AND NRPROGOCORRENCIA = $nrprogocorrencia";
            } elseif ($nrtipoprogocorrencia != '') {
                $condition = "AND NRTIPOPROGOCORRENCIA = $nrtipoprogocorrencia";
            } else {
                $condition = "AND 1 = 1";
            }

            $sql = <<<SQL
                DELETE FROM
                    PTO_PROGOCORRENCIA
                WHERE NRORG                = $nrorg
                    AND NRVINCULOM           = $nrvinculom
                    AND DTINICIOPROGOCOR     = TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY')
                    AND DTFIMPROGOCOR        = TO_DATE(:P_DTFIMPROGOCOR, 'DD/MM/YYYY')
                    $condition
SQL;
            
            $query = $this->connection->prepare($sql);

            $result = $query->execute(
                array(
                    "P_DTINICIOPROGOCOR"    => $dtinicioprogocor,
                    "P_DTFIMPROGOCOR"       => $dtfimprogocor
                )
            );

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     *  Função que retorna as ocorrências encontradas para um array de vínculos em um período específico
     *
     *  @param string $nrorg
     *  @param Array  $arrayNrvinculom
     *  @param string $nrperiodoapuracao
     *  @param string $idprogocorconfirmada
     * 
     *  @return array
     */
    public function getOcorrenciasPeriodo(string $nrorg, Array $arrayNrvinculom, string $nrperiodoapuracao, string $idprogocorconfirmada = null, $perfacesst = null) {
        try {            
            if (empty($nrperiodoapuracao)) {
                $periodo = $this->periodoRepository->getPeriodoApuracaoAtual($nrvinculom, $nrorg);
            } else {
                $periodo = $this->periodoRepository->getPeriodo($nrorg, $nrperiodoapuracao);
            }

            if (empty($periodo)) {
                throw new \Exception('Período não encontrado.');
            }

            $nrVinculoParam = implode(",", $arrayNrvinculom);

            $dtinicialapuracao = Utilities::formatDate($periodo['DTINICIALAPURACAO']);

            $dtfinalapuracao = Utilities::formatDate($periodo['DTFINALAPURACAO']);

            $sql = <<<SQL
                SELECT
                    PO.NRORG,
                    PO.NRPROGOCORRENCIA,
                    PO.NRVINCULOM,
                    VM.NMVINCULOM,
                    TO_CHAR(PO.DTINICIOPROGOCOR, 'DD/MM/YYYY') AS DTINICIOPROGOCOR,
                    TO_CHAR(PO.DTFIMPROGOCOR, 'DD/MM/YYYY') AS DTFIMPROGOCOR,
                    ((PO.DTFIMPROGOCOR - PO.DTINICIOPROGOCOR)  +1) AS DIAOCORRE,
                    PO.IDDIASSEMANA,
                    PO.IDTIPODIACALENDARIO,
                    PO.IDTIPOHORARIO,
                    PO.NRMOTIVOOCORFREQ,
                    PO.NRHORDIAM,
                    PO.NRSITUFREQUENCIA,
                    PO.NRSITUFREQGATILHO,
                    PO.QTMINGATILHOPROGOCOR,
                    PO.QTMAXGATILHOPROGOCOR,
                    PO.VRCOEFICIENTEACRESCIMO,
                    PO.IDPROGOCORRENCIA,
                    /*TRATATIVA DE QTMAXIMA E QTMINIMA*/
                    CASE
                        WHEN PO.QTMINIMAPROGOCOR IS NOT NULL THEN
                            SUBSTR(LPAD(PO.QTMINIMAPROGOCOR, 4, '0'), 1, 2) || ':' || SUBSTR(LPAD(PO.QTMINIMAPROGOCOR, 4, '0'), 3, 4)
                        ELSE
                            NULL
                    END AS QTMINIMAPROGOCOR,
                    CASE
                        WHEN PO.QTMAXIMAPROGOCOR IS NOT NULL THEN
                            SUBSTR(LPAD(PO.QTMAXIMAPROGOCOR, 4, '0'), 1, 2) || ':' || SUBSTR(LPAD(PO.QTMAXIMAPROGOCOR, 4, '0'), 3, 4)
                        ELSE
                            NULL
                    END AS QTMAXIMAPROGOCOR,
                    CASE
                        WHEN GPE_PARAMORG.IDAPROVOCORRENCIA = 'S' AND MO.IDGESTORAPROVA = 'S' THEN 'S'
                        WHEN :P_PERFACESST = 'S' THEN 'S'
                        ELSE
                            'N'
                    END AS ID_OPER_APROVA_OCOR,
                    PO.DSOBSERVACAO,
                    PO.IDATIVO,
                    PO.NRORGINCLUSAO,
                    PO.DTINCLUSAO,
                    PO.CDOPERINCLUSAO,
                    OP_INC.NMOPERADOR AS NMOPERINCLUSAO,
                    PO.NRORGULTATU,
                    PO.DTULTATU,
                    PO.CDOPERULTATU,
                    OP_ATU.NMOPERADOR AS NMOPERULTATU,
                    PO.NRTIPOPROGOCORRENCIA,
                    PO.NRMARCACAOPONTO,
                    PO.NRINTERVALOHORARIO,
                    PO.IDPROGOCORCONFIRMADA,
                    PO.CDOPERVERIFICADO,
                    PO.DSOBSERVACAORECUSA,
                    PO.HRPROJETARINTERVALONPROG,
                    PO.CDTABECDI,
                    PO.CDDIAGNOST,
                    PO.NRPESSOA,
                    DIAGNOST.NMDIAGNOST,
                    GPE_PESSOAH.NMPESSOA,
                    MO.DSMOTIVOOCORFREQ,
                    SF.DSSITUFREQUENCIA,
                    TP.NMTIPOPROGOCORRENCIA,
                    TP.IDPROGOCORACONFIRMAR,
                    PO.DSPATH,
                    TO_CHAR(MP.DTHRMARCACAOPONTO, 'HH24:MI') HRINICIO,
                    DECODE (PO.IDPROGOCORCONFIRMADA, 'S', 'Confirmada',
                                    'N', 'Rejeitada',
                                    'P', 'Pendente',
                                    'Pendente') DSCONFIRM
                FROM PTO_PROGOCORRENCIA PO
                LEFT JOIN PTO_MOTIVOOCORFREQ MO ON MO.NRORG = PO.NRORG AND MO.NRMOTIVOOCORFREQ = PO.NRMOTIVOOCORFREQ
                LEFT JOIN PTO_SITUFREQUENCIA SF ON SF.NRORG = PO.NRORG AND SF.NRSITUFREQUENCIA = PO.NRSITUFREQUENCIA
                LEFT JOIN GPE_VINCULOM VM ON VM.NRORG = PO.NRORG AND VM.NRVINCULOM = PO.NRVINCULOM
                LEFT JOIN PTO_TIPOPROGOCORRENCIA TP ON (TP.NRORG = PO.NRORG OR TP.NRORG = 0) AND TP.NRTIPOPROGOCORRENCIA = PO.NRTIPOPROGOCORRENCIA
                LEFT JOIN GPE_MARCACAOPONTO MP ON MP.NRORG = PO.NRORG AND MP.NRMARCACAOPONTO = PO.NRMARCACAOPONTO
                LEFT JOIN OPERADOR OP_INC ON OP_INC.NRORG = PO.NRORG
                    AND OP_INC.CDOPERADOR = PO.CDOPERINCLUSAO
                LEFT JOIN OPERADOR OP_ATU ON OP_ATU.NRORG = PO.NRORG
                    AND OP_ATU.CDOPERADOR = PO.CDOPERULTATU
                LEFT JOIN GPE_PARAMORG ON GPE_PARAMORG.NRORG = PO.NRORG
                
                /* DOENCA */
                LEFT JOIN DIAGNOST ON DIAGNOST.CDTABECDI = PO.CDTABECDI AND DIAGNOST.CDDIAGNOST = PO.CDDIAGNOST
                
                /* MEDICO */
                LEFT JOIN GPE_PESSOAH ON GPE_PESSOAH.NRPESSOA = PO.NRPESSOA
                                        AND GPE_PESSOAH.NRORG = PO.NRORG
                                        AND GPE_PESSOAH.DTMESCOMPETENC = (
                                                        (SELECT 
                                                           DTMESCOMPETENC 
                                                         FROM 
                                                           (SELECT 
                                                              H1.DTMESCOMPETENC 
                                                            FROM 
                                                              GPE_PESSOAH H1
                                                            WHERE 
                                                              H1.NRORG = GPE_PESSOAH.NRORG 
                                                              AND H1.NRPESSOA = GPE_PESSOAH.NRPESSOA 
                                                            ORDER BY H1.DTMESCOMPETENC DESC) 
                                                    WHERE ROWNUM = 1)
                )
                WHERE PO.NRORG = :P_NRORG
                    AND ((:P_PERFACESST = 'S' AND PO.NRMOTIVOOCORFREQ IN (SELECT MSF.NRMOTIVOOCORFREQ FROM PTO_MOTIVOSITUFUNC MSF WHERE MSF.NRORG = PO.NRORG))
                            OR (INSTR(',' ||:P_NRVINCULOM  || ',', ',' || PO.NRVINCULOM || ',') > 0))
                    AND (PO.IDPROGOCORCONFIRMADA = :P_IDPROGOCORCONFIRMADA OR :P_IDPROGOCORCONFIRMADA IS NULL)
                    AND PO.DTINICIOPROGOCOR BETWEEN TO_DATE(:P_DTINICIALAPURACAO, 'DD/MM/YYYY') AND TO_DATE(:P_DTFINALAPURACAO, 'DD/MM/YYYY')
                    AND (TP.NRTIPOPROGOCORRENCIA IN (1,4,5,10,13,16,17,22,23,24,25,30) OR TP.NRTIPOPROGOCORRENCIA IS NULL)
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);
            
            $query->execute(
                array(
                    "P_NRORG"                   => $nrorg,
                    "P_NRVINCULOM"              => $nrVinculoParam,
                    "P_DTINICIALAPURACAO"       => $dtinicialapuracao,
                    "P_DTFINALAPURACAO"         => $dtfinalapuracao,
                    "P_IDPROGOCORCONFIRMADA"    => $idprogocorconfirmada,
                    "P_PERFACESST"              => $perfacesst
                )
            );
            
            $resultQuery = $query->fetchAll();


            if (isset($resultQuery)) {
                return $resultQuery;
            } else {
                return null;
            }
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getMotivoOcorFreqAdmin(string $nrorg) {
        try {
            $sql = <<<SQL
                SELECT DSMOTIVOOCORFREQ,
                       NRMOTIVOOCORFREQ
                FROM PTO_MOTIVOOCORFREQ
                WHERE NRORG = :P_NRORG
                ORDER BY DSMOTIVOOCORFREQ
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG" => $nrorg
                )
            );

            $resultQuery = $query->fetchAll();

            if (isset($resultQuery)) {
                return $resultQuery;
            } else {
                return null;
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getMotivoOcorFreq(string $nrorg) {
        try {
            $sql = <<<SQL
                SELECT DSMOTIVOOCORFREQ,
                       NRMOTIVOOCORFREQ
                FROM PTO_MOTIVOOCORFREQ
                WHERE NRORG = :P_NRORG AND
                ((NRORG = 2973 AND NRMOTIVOOCORFREQ NOT IN (14 , 2046)) OR NRORG <> 2973)
                ORDER BY DSMOTIVOOCORFREQ
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG" => $nrorg
                )
            );

            $resultQuery = $query->fetchAll();

            if (isset($resultQuery)) {
                return $resultQuery;
            } else {
                return null;
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getMotivoOcorFreqLancamentoRetroativoEspecificasTeknisa(string $nrorg) {
        try {
            $sql = <<<SQL
                SELECT DSMOTIVOOCORFREQ,
                       NRMOTIVOOCORFREQ
                FROM PTO_MOTIVOOCORFREQ
                WHERE NRORG = :P_NRORG
                    AND NRMOTIVOOCORFREQ IN (36, 37)
                ORDER BY DSMOTIVOOCORFREQ
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG" => $nrorg
                )
            );

            $resultQuery = $query->fetchAll();

            if (isset($resultQuery)) {
                return $resultQuery;
            } else {
                return null;
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getTpOcorrencia($nrorg)  {
        try {
            $sql = <<<SQL
                SELECT
                    TP.NRTIPOPROGOCORRENCIA,
                    TP.NMTIPOPROGOCORRENCIA,
                    TP.IDPROGOCORACONFIRMAR
                FROM PTO_TIPOPROGOCORRENCIA TP
                WHERE (TP.NRORG = :P_NRORG OR TP.NRORG = 0)
                AND TP.NRTIPOPROGOCORRENCIA IN (1,2,3,4,5,10,13,16,17,22,23,24,25,29,30)
            ORDER BY TP.NMTIPOPROGOCORRENCIA
SQL;
            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG" => $nrorg
                )
            );

            $result = $query->fetchAll();

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function newOcorrenciaFreq($row) {
        try {
            $nrorg = $row['NRORG'];
            $nrprogocorrencia = NovoCodigo::geraCodigo($nrorg, 'PTO_PROGOCORRENCIA', 1, 22, true);

            $sql = <<<SQL
                INSERT INTO PTO_PROGOCORRENCIA (
                    NRPROGOCORRENCIA,
                    NRVINCULOM,
                    DTINICIOPROGOCOR,
                    DTFIMPROGOCOR,
                    IDTIPOHORARIO,
                    NRMOTIVOOCORFREQ,
                    DSOBSERVACAO,
                    NRTIPOPROGOCORRENCIA,
                    IDPROGOCORCONFIRMADA,
                    NRMARCACAOPONTO,
                    DTINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORGINCLUSAO,
                    NRORG,
                    DSPATH,
                    IDDIASSEMANA,
                    IDTIPODIACALENDARIO,
                    QTMINIMAPROGOCOR,
                    QTMAXIMAPROGOCOR,
                    CDTABECDI,
                    CDDIAGNOST,
                    NRPESSOA,
                    NRHORDIAM
                ) VALUES (
                    :P_NRPROGOCORRENCIA,
                    :P_NRVINCULOM,
                    TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY'),
                    TO_DATE(:P_DTFIMPROGOCOR, 'DD/MM/YYYY'),
                    :P_IDTIPOHORARIO,
                    :P_NRMOTIVOOCORFREQ,
                    :P_DSOBSERVACAO,
                    :P_NRTIPOPROGOCORRENCIA,
                    :P_IDPROGOCORCONFIRMADA,
                    :P_NRMARCACAOPONTO,
                    :P_DTINCLUSAO,
                    :P_CDOPERINCLUSAO,
                    :P_NRORG,
                    :P_NRORG,
                    :P_DSPATH,
                    :P_IDDIASSEMANA,
                    :P_IDTIPODIACALENDARIO,
                    :P_QTMINIMAPROGOCOR,
                    :P_QTMAXIMAPROGOCOR,
                    :P_CDTABECDI,
                    :P_CDDIAGNOST,
                    :P_NRPESSOA,
                    :P_NRHORDIAM
                )
SQL;

            $params = [
                "P_NRPROGOCORRENCIA"            => $nrprogocorrencia,
                "P_NRVINCULOM"                  => isset($row['NRVINCULOM'])            ? $row['NRVINCULOM'] : NULL ,
                "P_DTINICIOPROGOCOR"            => isset($row['DTINICIOPROGOCOR'])      ? $row['DTINICIOPROGOCOR'] : NULL ,
                "P_DTFIMPROGOCOR"               => isset($row['DTFIMPROGOCOR'])         ? $row['DTFIMPROGOCOR'] : NULL ,
                "P_IDTIPOHORARIO"               => isset($row['IDTIPOHORARIO'])         ? $row['IDTIPOHORARIO'] : 'N' ,
                "P_NRMOTIVOOCORFREQ"            => isset($row['NRMOTIVOOCORFREQ'])      ? $row['NRMOTIVOOCORFREQ'] : NULL ,
                "P_DSOBSERVACAO"                => isset($row['DSOBSERVACAO'])          ? $row['DSOBSERVACAO'] : NULL ,
                "P_NRTIPOPROGOCORRENCIA"        => isset($row['NRTIPOPROGOCORRENCIA'])  ? $row['NRTIPOPROGOCORRENCIA'] : NULL ,
                "P_IDPROGOCORCONFIRMADA"        => isset($row['IDPROGOCORCONFIRMADA'])  ? $row['IDPROGOCORCONFIRMADA'] : 'P' ,
                "P_NRMARCACAOPONTO"             => isset($row['NRMARCACAOPONTO'])       ? $row['NRMARCACAOPONTO'] : NULL ,
                "P_DTINCLUSAO"                  => isset($row['DTINCLUSAO'])            ? $row['DTINCLUSAO'] : date("d/m/Y H:i:s") ,
                "P_CDOPERINCLUSAO"              => isset($row['CDOPERADOR'])            ? $row['CDOPERADOR'] : Utils::getParamters()->getCdOperador(),
                "P_DSPATH"                      => isset($row['DSPATH'])                ? $row['DSPATH'] : NULL,
                "P_NRORG"                       => $nrorg,
                "P_IDDIASSEMANA"                => isset($row['IDDIASSEMANA'])          ? $row['IDDIASSEMANA'] : NULL,
                "P_IDTIPODIACALENDARIO"         => isset($row['IDTIPODIACALENDARIO'])   ? $row['IDTIPODIACALENDARIO'] : NULL,
                "P_QTMINIMAPROGOCOR"            => isset($row['QTMINIMAPROGOCOR'])      ? str_replace(':', '', $row['QTMINIMAPROGOCOR']) : NULL,
                "P_QTMAXIMAPROGOCOR"            => isset($row['QTMAXIMAPROGOCOR'])      ? str_replace(':', '', $row['QTMAXIMAPROGOCOR']) : NULL,
                "P_CDTABECDI"                   => isset($row['CDTABECDI'])             ? str_replace(':', '', $row['CDTABECDI'])        : NULL,
                "P_CDDIAGNOST"                  => isset($row['CDDIAGNOST'])            ? str_replace(':', '', $row['CDDIAGNOST'])       : NULL,
                "P_NRPESSOA"                    => isset($row['NRPESSOA'])              ? str_replace(':', '', $row['NRPESSOA'])         : NULL,
                "P_NRHORDIAM"                   => isset($row['NRHORDIAM'])             ? str_replace(':', '', $row['NRHORDIAM'])        : NULL,
            ];

            $query = $this->entityManager->getConnection()->prepare($sql);
            $result = $query->execute($params);
            
            return $result ? $nrprogocorrencia : null;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updateOcorrenciaFreq($row) {
        try {
            $nrorg = $row['NRORG'];

            $sql = <<<SQL
                UPDATE PTO_PROGOCORRENCIA
                    SET
                        DTINICIOPROGOCOR      =  TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY'),
                        DTFIMPROGOCOR         =  TO_DATE(:P_DTFIMPROGOCOR, 'DD/MM/YYYY'),
                        IDTIPOHORARIO         =  :P_IDTIPOHORARIO,
                        NRMOTIVOOCORFREQ      =  :P_NRMOTIVOOCORFREQ,
                        DSOBSERVACAO          =  :P_DSOBSERVACAO,
                        NRTIPOPROGOCORRENCIA  =  :P_NRTIPOPROGOCORRENCIA,
                        IDPROGOCORCONFIRMADA  =  :P_IDPROGOCORCONFIRMADA,
                        QTMINIMAPROGOCOR      =  :P_QTMINIMAPROGOCOR,
                        QTMAXIMAPROGOCOR      =  :P_QTMAXIMAPROGOCOR,
                        CDTABECDI             =  :P_CDTABECDI,
                        CDDIAGNOST            =  :P_CDDIAGNOST,
                        NRPESSOA              =  :P_NRPESSOA,
                        NRHORDIAM             =  :P_NRHORDIAM,
                        CDOPERULTATU          =  :P_CDOPERULTATU,
                        DTULTATU              =  :P_DTULTATU,
                        NRORGULTATU           =  :P_NRORG
                    WHERE
                        NRPROGOCORRENCIA = :P_NRPROGOCORRENCIA AND NRORG = :P_NRORG
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $result = $query->execute(
                array(
                    "P_DTINICIOPROGOCOR"            => isset($row['DTINICIOPROGOCOR'])      ? $row['DTINICIOPROGOCOR'] : NULL ,
                    "P_DTFIMPROGOCOR"               => isset($row['DTFIMPROGOCOR'])         ? $row['DTFIMPROGOCOR'] : NULL ,
                    "P_IDTIPOHORARIO"               => isset($row['IDTIPOHORARIO'])         ? $row['IDTIPOHORARIO'] : 'N' ,
                    "P_NRMOTIVOOCORFREQ"            => isset($row['NRMOTIVOOCORFREQ'])      ? $row['NRMOTIVOOCORFREQ'] : NULL ,
                    "P_DSOBSERVACAO"                => isset($row['DSOBSERVACAO'])          ? $row['DSOBSERVACAO'] : NULL ,
                    "P_NRTIPOPROGOCORRENCIA"        => isset($row['NRTIPOPROGOCORRENCIA'])  ? $row['NRTIPOPROGOCORRENCIA'] : NULL ,
                    "P_IDPROGOCORCONFIRMADA"        => isset($row['IDPROGOCORCONFIRMADA'])  ? $row['IDPROGOCORCONFIRMADA'] : NULL ,
                    "P_NRPROGOCORRENCIA"            => isset($row['NRPROGOCORRENCIA'])      ? $row['NRPROGOCORRENCIA'] : NULL ,
                    "P_QTMINIMAPROGOCOR"            => isset($row['QTMINIMAPROGOCOR'])      ? str_replace(':', '', $row['QTMINIMAPROGOCOR']) : NULL,
                    "P_QTMAXIMAPROGOCOR"            => isset($row['QTMAXIMAPROGOCOR'])      ? str_replace(':', '', $row['QTMAXIMAPROGOCOR']) : NULL,
                    "P_CDTABECDI"                   => isset($row['CDTABECDI'])             ? str_replace(':', '', $row['CDTABECDI'])        : NULL,
                    "P_CDDIAGNOST"                  => isset($row['CDDIAGNOST'])            ? str_replace(':', '', $row['CDDIAGNOST'])       : NULL,
                    "P_NRPESSOA"                    => isset($row['NRPESSOA'])              ? str_replace(':', '', $row['NRPESSOA'])         : NULL,
                    "P_NRHORDIAM"                   => isset($row['NRHORDIAM'])             ? str_replace(':', '', $row['NRHORDIAM'])        : NULL,
                    "P_NRORG"                       => $nrorg,
                    "P_DTULTATU"                    => isset($row['DTULTATU'])              ? $row['DTULTATU'] : date("d/m/Y H:i:s") ,
                    "P_CDOPERULTATU"                => isset($row['CDOPERADOR'])            ? $row['CDOPERADOR'] : Utils::getParamters()->getCdOperador(),
                )
            );
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function removeOcorrenciaFreq($row) {
        try {
            $nrorg = $row['NRORG'];

            $sql = <<<SQL
                DELETE FROM PTO_PROGOCORRENCIA
                    WHERE
                    NRPROGOCORRENCIA = :P_NRPROGOCORRENCIA AND NRORG = :P_NRORG
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $result = $query->execute(
                array(
                    "P_NRPROGOCORRENCIA" => isset($row['NRPROGOCORRENCIA'])      ? $row['NRPROGOCORRENCIA'] : NULL ,
                    "P_NRORG"            => $nrorg
                )
            );
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getOcorrenciasPorTipoOcorrencia(
        string  $nrvinculom,
        string  $nrorg,
        string  $nrtipoprogocorrencia,
        string  $dtinicioprogocor   = null,
        string  $nrmarcacaoponto    = null
    ): array {
        try {
            $sql = <<<SQL
                SELECT
                    *
                FROM
                    PTO_PROGOCORRENCIA OCOR
                WHERE
                    OCOR.NRORG                      = :P_NRORG
                    AND OCOR.NRTIPOPROGOCORRENCIA   = :P_NRTIPOPROGOCORRENCIA
                    AND OCOR.NRVINCULOM             = :P_NRVINCULOM
                    AND (TO_CHAR(OCOR.DTINICIOPROGOCOR, 'DD/MM/YYYY')   = :P_DTINICIOPROGOCOR   OR :P_DTINICIOPROGOCOR  IS NULL)
                    AND (OCOR.NRMARCACAOPONTO                           = :P_NRMARCACAOPONTO    OR :P_NRMARCACAOPONTO   IS NULL)
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    'P_NRORG'                   => $nrorg,
                    'P_NRTIPOPROGOCORRENCIA'    => $nrtipoprogocorrencia,
                    'P_NRVINCULOM'              => $nrvinculom,
                    'P_DTINICIOPROGOCOR'        => $dtinicioprogocor,
                    'P_NRMARCACAOPONTO'         => $nrmarcacaoponto
                )
            );

            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getOcorrenciasDia(
        string  $nrorg,
        string  $nrvinculom,
                $dtinicioprogocor,
        string  $nrtipoprogocorrencia = null,
        string  $perfacesst           = null
    ){
        
        try {

            $sql = <<<SQL
                SELECT
                    PO.NRORG,
                    PO.NRPROGOCORRENCIA,
                    PO.NRVINCULOM,
                    TO_CHAR(PO.DTINICIOPROGOCOR, 'DD/MM/YYYY') AS DTINICIOPROGOCOR,
                    TO_CHAR(PO.DTFIMPROGOCOR, 'DD/MM/YYYY') AS DTFIMPROGOCOR,
                    ((PO.DTFIMPROGOCOR - PO.DTINICIOPROGOCOR)  +1) AS DIAOCORRE,
                    PO.IDDIASSEMANA,
                    PO.IDTIPODIACALENDARIO,
                    PO.IDTIPOHORARIO,
                    PO.NRMOTIVOOCORFREQ,
                    PO.NRHORDIAM,
                    PO.NRSITUFREQUENCIA,
                    PO.NRSITUFREQGATILHO,
                    PO.QTMINGATILHOPROGOCOR,
                    PO.QTMAXGATILHOPROGOCOR,
                    PO.VRCOEFICIENTEACRESCIMO,
                    PO.IDPROGOCORRENCIA,
                    /*TRATATIVA DE QTMAXIMA E QTMINIMA*/
                    CASE
                        WHEN PO.QTMINIMAPROGOCOR IS NOT NULL THEN
                            SUBSTR(LPAD(PO.QTMINIMAPROGOCOR, 4, '0'), 1, 2) || ':' || SUBSTR(LPAD(PO.QTMINIMAPROGOCOR, 4, '0'), 3, 4)
                        ELSE
                            NULL
                    END AS QTMINIMAPROGOCOR,
                    CASE
                        WHEN PO.QTMAXIMAPROGOCOR IS NOT NULL THEN
                            SUBSTR(LPAD(PO.QTMAXIMAPROGOCOR, 4, '0'), 1, 2) || ':' || SUBSTR(LPAD(PO.QTMAXIMAPROGOCOR, 4, '0'), 3, 4)
                        ELSE
                            NULL
                    END AS QTMAXIMAPROGOCOR,
                    CASE
                        WHEN :P_NRPERFILSESMT IS NOT NULL THEN 'S'
                        WHEN GPE_PARAMORG.IDAPROVOCORRENCIA = 'S' AND MO.IDGESTORAPROVA = 'S' THEN 'S'
                        ELSE
                            'N'
                    END AS ID_OPER_APROVA_OCOR,
                    PO.DSOBSERVACAO,
                    PO.IDATIVO,
                    PO.NRORGINCLUSAO,
                    PO.DTINCLUSAO,
                    PO.CDOPERINCLUSAO,
                    OP_INC.NMOPERADOR AS NMOPERINCLUSAO,
                    PO.NRORGULTATU,
                    PO.DTULTATU,
                    PO.CDOPERULTATU,
                    OP_ATU.NMOPERADOR AS NMOPERULTATU,
                    PO.NRTIPOPROGOCORRENCIA,
                    PO.NRMARCACAOPONTO,
                    PO.NRINTERVALOHORARIO,
                    PO.IDPROGOCORCONFIRMADA,
                    PO.CDOPERVERIFICADO,
                    PO.DSOBSERVACAORECUSA,
                    PO.HRPROJETARINTERVALONPROG,
                    PO.CDTABECDI,
                    PO.CDDIAGNOST,
                    PO.NRPESSOA,
                    DIAGNOST.NMDIAGNOST,
                    GPE_PESSOAH.NMPESSOA,
                    MO.DSMOTIVOOCORFREQ,
                    SF.DSSITUFREQUENCIA,
                    TP.NMTIPOPROGOCORRENCIA,
                    TP.IDPROGOCORACONFIRMAR,
                    PO.DSPATH,
                    DECODE (PO.IDPROGOCORCONFIRMADA, 'S', 'Confirmada',
                                     'N', 'Rejeitada',
                                     'P', 'Pendente',
                                     'Pendente') DSCONFIRM
                FROM PTO_PROGOCORRENCIA PO
                LEFT JOIN PTO_MOTIVOOCORFREQ MO ON MO.NRORG = PO.NRORG AND MO.NRMOTIVOOCORFREQ = PO.NRMOTIVOOCORFREQ
                LEFT JOIN PTO_SITUFREQUENCIA SF ON SF.NRORG = PO.NRORG AND SF.NRSITUFREQUENCIA = PO.NRSITUFREQUENCIA
                LEFT JOIN PTO_TIPOPROGOCORRENCIA TP ON (TP.NRORG = PO.NRORG OR TP.NRORG = 0) AND TP.NRTIPOPROGOCORRENCIA = PO.NRTIPOPROGOCORRENCIA
                LEFT JOIN GPE_PARAMORG ON GPE_PARAMORG.NRORG = PO.NRORG
                LEFT JOIN OPERADOR OP_INC ON OP_INC.NRORG = PO.NRORG
                    AND OP_INC.CDOPERADOR = PO.CDOPERINCLUSAO
                LEFT JOIN OPERADOR OP_ATU ON OP_ATU.NRORG = PO.NRORG
                    AND OP_ATU.CDOPERADOR = PO.CDOPERULTATU
                
                /* DOENCA */
                LEFT JOIN DIAGNOST ON DIAGNOST.CDTABECDI = PO.CDTABECDI AND DIAGNOST.CDDIAGNOST = PO.CDDIAGNOST
                
                /* MEDICO */
                LEFT JOIN GPE_PESSOAH ON GPE_PESSOAH.NRPESSOA = PO.NRPESSOA
                                         AND GPE_PESSOAH.NRORG = PO.NRORG
                                         AND GPE_PESSOAH.DTMESCOMPETENC = (
                    SELECT
                        MAX(H1.DTMESCOMPETENC)
                    FROM
                        GPE_PESSOAH H1
                    WHERE
                        H1.NRPESSOA = GPE_PESSOAH.NRPESSOA
                        AND H1.NRORG = GPE_PESSOAH.NRORG
                        AND H1.DTMESCOMPETENC <= :P_DTINICIOPROGOCOR
                )
                WHERE PO.NRORG = :P_NRORG
                    AND (PO.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
                    AND PO.DTINICIOPROGOCOR = TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY')
                    AND (PO.NRTIPOPROGOCORRENCIA = :P_NRTIPOPROGOCORRENCIA OR :P_NRTIPOPROGOCORRENCIA IS NULL)
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"                   => $nrorg,
                    "P_NRVINCULOM"              => $nrvinculom,
                    "P_DTINICIOPROGOCOR"        => $dtinicioprogocor,
                    "P_NRTIPOPROGOCORRENCIA"    => $nrtipoprogocorrencia,
                    "P_NRPERFILSESMT"           => $perfacesst
                )
            );

            $resultQuery = $query->fetchAll();

            if (isset($resultQuery)) {
                return $resultQuery;
            } else {
                return null;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getAlmoco(array $params) {
        $sql = <<<SQL
            SELECT *
            FROM PTO_PROGOCORRENCIA
            WHERE NRTIPOPROGOCORRENCIA IN (20, 26)
              AND NRORG = :P_NRORG
              AND NRVINCULOM = :P_NRVINCULO
              AND DTINICIOPROGOCOR = :P_DATE
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $almoco = $std->fetch();

        return $almoco;
    }
    
    public function getOcorrenciaByNrmarcacao(
        $nrorg,
        $nrmarcacaoponto
    ) {
        $sql = <<<SQL
            SELECT *
            FROM PTO_PROGOCORRENCIA
            WHERE NRORG             = :P_NRORG
              AND NRMARCACAOPONTO   = :P_NRMARCACAOPONTO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRMARCACAOPONTO' => $nrmarcacaoponto
            )    
        );

        $ocorrencia = $std->fetch();

        return $ocorrencia;
    }
    
    public function removeOcorrenciaByNrmarcacao(
        $nrorg,
        $nrmarcacaoponto
    ) {
        $sql = <<<SQL
            DELETE FROM
                PTO_PROGOCORRENCIA
            WHERE NRORG                 = :P_NRORG
                AND NRMARCACAOPONTO     = :P_NRMARCACAOPONTO
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $result = $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRMARCACAOPONTO' => $nrmarcacaoponto
            )
        );

        return $result;
    }
    
    public function updateOcorrenciaAlmoco(
        $nrorg,
        $nrmarcacaoponto,
        $nrtipoprogocorrencia,
        $hrprojetarintervalonprog    
    ) {
        $sql = <<<SQL
            UPDATE PTO_PROGOCORRENCIA
                SET NRTIPOPROGOCORRENCIA        = :P_NRTIPOPROGOCORRENCIA,
                    HRPROJETARINTERVALONPROG    = :P_HRPROJETARINTERVALONPROG
                WHERE NRMARCACAOPONTO = :P_NRMARCACAOPONTO
                    AND NRORG = :P_NRORG
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $result = $query->execute(
            array(
                'P_NRORG'                       => $nrorg,
                'P_NRMARCACAOPONTO'             => $nrmarcacaoponto,
                'P_NRTIPOPROGOCORRENCIA'        => $nrtipoprogocorrencia,
                'P_HRPROJETARINTERVALONPROG'    => $hrprojetarintervalonprog,
            )
        );

        return $result;
    }

    public function getOcorrenciasAdmissao(
        int $nrOrg,
        int $nrTipoVinculo
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                PTO_OCORRENCIADMISSAO
            WHERE
                NRORG = :P_NRORG AND
                (
                    NRTIPOVINCULOM = :P_NRTIPOVINCULOM OR
                    :P_NRTIPOVINCULOM IS NULL
                )
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRTIPOVINCULOM' => $nrTipoVinculo,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getMotivo(
        int $nrOrg,
        int $nrMotivo
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                PTO_MOTIVOOCORFREQ
            WHERE
                NRORG = :P_NRORG AND
                NRMOTIVOOCORFREQ = :P_NRMOTIVO
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRMOTIVO' => $nrMotivo,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $motivo = $std->fetch();

        return $motivo;
    }

    public function getEventosAtivos(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                EVM.DTINIVIGENCIA,
                EVM.DTFIMVIGENCIA,
                EVM.IDTIPOEVENTO,
                EVH.*, NR.CDESOCIAL,
                NR.NMNATURUBR
            FROM
                FPA_EVENTOM EVM
                INNER JOIN
                FPA_EVENTOH EVH ON
                    (
                        EVH.NREVENTOM = EVM.NREVENTOM AND
                        EVH.NRORG = EVM.NRORG
                    ) AND
                    EVH.DTMESCOMPETENC = (
                        SELECT
                            MAX(EVTH.DTMESCOMPETENC)
                        FROM
                            FPA_EVENTOH EVTH
                        WHERE
                            EVTH.NRORG = EVH.NRORG AND
                            EVTH.NREVENTOM = EVH.NREVENTOM AND
                            EVTH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                        )
                LEFT JOIN
                NATUREZARUBRICA NR ON
                    NR.NRNATURUBR = EVH.NRNATURUBR
            WHERE
                (
                    EVM.NRORG = :P_NRORG OR
                    EVM.NRORG = :P_NRORG_PADRAO
                ) AND
                EVM.DTINIVIGENCIA <= :P_DTMESCOMPETENC AND
                (
                    EVM.DTFIMVIGENCIA >= :P_DTMESCOMPETENC OR
                    EVM.DTFIMVIGENCIA IS NULL
                )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }
    
    public function getDiagnost(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                NRORG,
                CDTABECDI,
                CDDIAGNOST,
                NMDIAGNOST,
                IDRESTRSEXO,
                CDTABECDI || ' | ' || CDDIAGNOST || ' | ' || NMDIAGNOST AS NMDIAGNOST_VIEW
            FROM
                DIAGNOST
            WHERE
                (
                    DIAGNOST.NRORG = :P_NRORG OR
                    DIAGNOST.NRORG = :P_NRORG_PADRAO
                )
            ORDER BY DIAGNOST.NMDIAGNOST
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }
    
    public function getPessoaConselhoReg(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                PH.NRORG,
                PH.NMPESSOA,
                PH.NRPESSOA,
                PH.NRPESSOAH,
                PH.NRCONSELHOREG,
                PH.NRINSCRCONSREG,
                PH.DTEXCONSREG,
                PH.SGUFCONSREG,
                GPE_CONSELHOREG.NMCONSELHOREG,
                PH.NMPESSOA || ' | ' || GPE_CONSELHOREG.NMCONSELHOREG NMPESSOA_VIEW
            FROM
                GPE_PESSOA    PM
                JOIN GPE_PESSOAH   PH ON PH.NRPESSOA = PM.NRPESSOA
                                       AND PH.NRORG = PM.NRORG
                                       AND PH.DTMESCOMPETENC = (
                    SELECT
                        MAX(H1.DTMESCOMPETENC)
                    FROM
                        GPE_PESSOAH H1
                    WHERE
                        H1.NRPESSOA = PH.NRPESSOA
                        AND H1.NRORG = PH.NRORG
                        AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                )
                LEFT JOIN GPE_CONSELHOREG ON ( GPE_CONSELHOREG.NRORG = PM.NRORG
                                               OR GPE_CONSELHOREG.NRORG = :P_NRORG_PADRAO )
                                             AND GPE_CONSELHOREG.NRCONSELHOREG = PH.NRCONSELHOREG
            WHERE
                PM.NRORG = :P_NRORG
                AND PH.NRCONSELHOREG IS NOT NULL
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }
    
    public function getHoradiah(
        array $params    
    ) {
        $sql = <<<SQL
            SELECT
                EM.NRORG,
                EH.NRHORDIAH,
                EM.NRHORDIAM,
                EH.DTMESCOMPETENC,
                EM.DTINIVIGENCIA,
                EM.DTFIMVIGENCIA,
                EH.IDTIPOHORARIO,
                EH.HRINICOLETAMARCACAO,
                EH.NMHORDIAH
            FROM
                GPE_HORDIAM   EM
                JOIN GPE_HORDIAH   EH ON EM.NRORG = EH.NRORG
                                       AND EM.NRHORDIAM = EH.NRHORDIAM
            WHERE EH.DTMESCOMPETENC = (
                    SELECT
                        MAX(DTMESCOMPETENC)
                    FROM GPE_HORDIAH WHERE
                        NRORG = EH.NRORG
                        AND NRHORDIAM = EH.NRHORDIAM
                        AND DTMESCOMPETENC <= :P_DTMESCOMPETENC
                )
                AND ( EM.NRORG = :P_NRORG
                      OR EM.NRORG = :P_NRORG_PADRAO )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function verificaPerfilAcessoSESMT(string $nrorg, string $cdoperador) {
        try {
            $sql = <<<SQL
                SELECT 
                    1
                FROM 
                    OPERADOR OP
                    JOIN PERFACESSPARC PA
                        ON PA.NRORG = OP.NRORG
                            AND PA.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
                    LEFT JOIN GPE_PARAMORG PO
                        ON PO.NRORG = OP.NRORG
                WHERE 
                    OP.NRORG = :P_NRORG
                    AND OP.CDOPERADOR = :P_CDOPERADOR
                    AND PO.NRPERFACESSOSST = PA.NRPERFILACESSO
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"       => $nrorg,
                    "P_CDOPERADOR"  => $cdoperador
                )
            );

            $resultQuery = $query->fetchAll();

            if (!empty($resultQuery)) {
                return 'S';
            } else {
                return null;
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }
}