<?php

namespace HCM\FPA\Entity;

use HCM\Util\Repositories;
use HCM\Util\MsgCalculo;
use HCM\Util\TkEntity;
use HCM\Util\FPA;
use HCM\FPA\Entity\FpaeventohEntity;
use HCM\FPA\Calculos\Geral\MediaEventosIntervalo;

class FpaeventomEntity extends TkEntity
{

    private $_nrOrg;
    private $_nrOrgPadrao;
    public $_mesCompetencia;
    private $_dtPagamento;
    private $_tipoEvento;
    private $_historico;

    public $preservar;

    const IDTIPOEVENTO_PROVENTO = 'PROVENTO';

    const IDTIPOEVENTO_DESCONTO = 'DESCONTO';

    const IDTIPOEVENTO_ENCARGO = 'ENCARGO';

    const IDTIPOEVENTO_TOTAL = 'TOTAL';
    
    public function __construct(
        $entityManager, $hcmParameters, $dtPagamento = null, $dtCompetencia = null
    ){
        $this->_nrOrg = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $hcmParameters->getNrorgPadrao();
        $this->_mesCompetencia = $hcmParameters->getDtCompetencia();
        $this->_dtPagamento = $dtPagamento;

        $this->_tipoEvento = array(
                              self::IDTIPOEVENTO_PROVENTO => '1',
                              self::IDTIPOEVENTO_DESCONTO => '2',
                              self::IDTIPOEVENTO_ENCARGO  => '3',
                              self::IDTIPOEVENTO_TOTAL    => '4',
                             );
        parent::__construct(null, $entityManager);
    }

    public function setMesCompetencia($mesCompetencia){
        $this->_mesCompetencia = $mesCompetencia;
    }


    public function setDataPagamento($dataPagamento){
        $this->_dtPagamento = $dataPagamento;
    }
    
    public function getDataPagamento(){
        return $this->_dtPagamento;
    }

    public function getOrdemTipoEvento($ordem) {
        return $this->_tipoEvento[$ordem];
    }

    private function _inicializaLista($nrOrg){
        $tmp = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEventos($nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);
        $this->preservar = array();
        foreach ($tmp as $tmp_evento) {
            
            if($tmp_evento->historico->getNrpriorestorno() == 999){
                $this->preservar[]=$tmp_evento->getNreventom();
            }
            $tmp_evento->_mesCompetencia = $this->_mesCompetencia;
            $tmp_evento->_dtPagamento = $this->_dtPagamento;
            $this->addItem($tmp_evento, $tmp_evento->getNreventom());
        }
        unset($tmp);
    }


    public function retornaEvento($nr_evento){
        if (empty($this->items) === true) {
            $this->_inicializaLista($this->_nrOrg);
        }
        return $this->items[$nr_evento];
    }

    private function _inicializaListaMedia(){
        $tmp = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaListaEventosMedia($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);
        
        foreach ($tmp as $tmp_evento) {
            $this->addItem($tmp_evento, $tmp_evento->getNreventom());
        }
    }

    public function retornaListaEventoMedia(){

        $itens = $this->getItems();

        if (empty($itens) === true) {
            $this->_inicializaListaMedia();
            $itens = $this->getItems();
        }

        $lista='';
        foreach ($itens as $item) {
            $lista = $lista.$item->getNreventom().',';
        }

            $lista = substr($lista, 0, strlen($lista)-1);


        return $lista;
    }

    private function _inicializaListaFalta($nrOrg, $nrOrgPadrao, $idFaltasFerias){
        $tmp = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaListaEventosFalta($nrOrg, $nrOrgPadrao, $idFaltasFerias, $this->_mesCompetencia);
        foreach ($tmp as $tmp_evento) {
            $this->addItem($tmp_evento, $tmp_evento->getNreventom());
        }
    }

    public function retornaListaEventoFalta($nrOrg, $nrOrgPadrao,$idFaltasFerias = null,$retornarEvento = false){
        $idFaltasFerias = $idFaltasFerias ? $idFaltasFerias : array(FpaeventohEntity::IDFALTASFERIAS_SOMA,FpaeventohEntity::IDFALTASFERIAS_SUBTRAI);
        $itens = $this->getItems();

        if (empty($itens) === true) {
            $this->_inicializaListaFalta($nrOrg, $nrOrgPadrao,$idFaltasFerias);
            $itens = $this->getItems();
            $lista='';
            if($retornarEvento){
                $lista = $itens;
            }else{
                foreach ($itens as $item) {
                    $lista = $lista.$item->getNreventom().',';
                }
                $lista = substr($lista, 0, strlen($lista)-1);
            }
        }
        $this->resetItems();
        return $lista;
    }

    public function novoMestre($dtIniVigencia, $idTipoEvento){
        try{
            $evento = new self(null, $this->_nrOrg, $this->_nrOrgPadrao);

            $evento->setDtinivigencia($dtIniVigencia);
            $evento->setIdtipoevento($idTipoEvento);
            $evento->setNrorg($this->_nrOrg);
            $evento->persist();

            $nrEvento = $evento->getNreventom();
            unset($evento);
            return $nrEvento;

        }
        catch (Exception $e){
            throw new Exception('Erro ao criar um novo evento. Nao pode inserir novo mestre.<br/>'.$e->getMessage());
        }

    }

    public function destroiMestre($nreventom, $nrorgTrab){
        try{
            $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento($nreventom, $nrorgTrab);
            if (is_object($evento) === true) {
                $this->addItem($evento, $nreventom);
                $this->removeItem($evento);
                $this->persist();
                return true;
            } else {
                return false;
            }
        }
        catch (Exception $e){
            return false;
        }

    }

    public function insereEventoMestre($nrorg, $idTipoEvento, $dtIniVigencia) {
        $this->setNrorg($nrorg);
        $this->setIdtipoevento($idTipoEvento);
        $this->setDtinivigencia($dtIniVigencia);
    }

    public static function getEventosArray($nrorg, $nrorgpadrao) {
        $lista = array();
        $eventos = self::getEventos($nrorg, $nrorgpadrao);
        foreach($eventos as $evento) {
            $lista[$evento->getNreventom()] = $evento;
        }
        return $lista;
    }

    public static function getEventos($nrorg, $nrorgpadrao) {
        $eventos = self::getEntitiesByWhere('NRORG = ? OR NRORG = ?', array($nrorg, $nrorgpadrao));
        return $eventos;
    }

    public function getEvento($nreventom) {
        $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento($nreventom, $this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);
        return $evento;
    }

    public function getHistorico() {
        if (!$this->_historico) {
            $this->_historico = FpaeventohEntity::getUltimoHistorico($this->getNreventom());
        }
        return $this->_historico;
    }
    
    public static function compatibilizaReferenciaHoras($referencia, $tipo_ref, $jornada_diaria){

        switch($tipo_ref){
        case FpaeventohEntity::IDUNIDREFEREVEN_HORA:
            return $referencia;
            break;
        case FpaeventohEntity::IDUNIDREFEREVEN_DIA:
            return $referencia * $jornada_diaria;
            break;
        case FpaeventohEntity::IDUNIDREFEREVEN_HORA_MINUTO:
            return FPA::converteHoraDecimal($referencia);
            break;
        default:
            $msg = 'Verifique o tipo de referencia: '.$tipo_ref;
            $msg .= ' Referencia:'.$referencia.' - '.get_called_class().'<br/>';
            MsgCalculo::getInstance()->adicionaMensagem($msg);
        }

    }

    public function compatibilizaReferenciaDiasParaN($dias, $jornada_diaria){

        switch($this->getIdunidrefereven()){
        case FpaeventohEntity::IDUNIDREFEREVEN_DIA:
            return $dias;
            break;
        case FpaeventohEntity::IDUNIDREFEREVEN_HORA:
            $referencia = $dias * $jornada_diaria;
            if ($referencia > 219.9 && $referencia < 220.01){
                $referencia = 220.0;
            } else if ($referencia > 109.9 && $referencia < 110.01){
                $referencia = 110.0;
            }
            return $referencia;
            break;
        default:
            $msg = 'Verifique o tipo de referencia: '.$tipo_ref;
            $msg .= ' Referencia:'.$referencia.' - '.get_called_class().'<br/>';
            MsgCalculo::getInstance()->adicionaMensagem($msg);
        }

    }

    public static function compatibilizaReferenciaDias($referencia, $tipo_ref, $jornada_diaria){
        switch($tipo_ref){
        case FpaeventohEntity::IDUNIDREFEREVEN_HORA:
            return $referencia / $jornada_diaria;
            break;
        case FpaeventohEntity::IDUNIDREFEREVEN_DIA:
            return $referencia;
            break;
        case FpaeventohEntity::IDUNIDREFEREVEN_HORA_MINUTO:
            $referenciaAux = FPA::converteHoraDecimal($referencia);
            return $referenciaAux / $jornada_diaria;
            break;
        default:
            $msg = 'Verifique o tipo de referencia: '.$tipo_ref;
            $msg .= ' Referencia:'.$referencia.' - '.get_called_class().'<br/>';
            MsgCalculo::getInstance()->adicionaMensagem($msg);
        }
    }
    
}