<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\Model\Gestaopessoas\Entities\FpaRegrah;
use Teknisa\Model\Gestaopessoas\Entities\FpaRegram;
use \HCM\FPA\Repository\RegraRepository;

class RegraService {
    
    protected $entityManager;
    protected $hcmParameters;
    protected $pontoRegraService;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, PontoRegraService $pontoRegraService, RegraRepository $regraRepository) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->pontoRegraService = $pontoRegraService;
        $this->regraRepository = $regraRepository;
    }
    
    public function deleteRegra($dataset) {
        try {
            $this->entityManager->beginTransaction();
            $regrah = $this->entityManager->getRepository(Repositories::FPA_REGRAH)->findOneBy(array("nrregram" => $dataset['NRREGRAM'], "nrregrah" => $dataset['NRREGRAH']));
            $this->entityManager->remove($regrah);
            $this->entityManager->flush();
            $regrahh = $this->entityManager->getRepository(Repositories::FPA_REGRAH)->findOneBy(array("nrregram" => $dataset['NRREGRAM']));
            if(empty($regrahh)) {
                $regram = $this->entityManager->getRepository(Repositories::FPA_REGRAM)->findOneBy(array("nrregram" => $dataset['NRREGRAM'],"nrorg" => $dataset['NRORG']));
                $this->entityManager->remove($regram);
                $this->entityManager->flush();
            }
            $this->entityManager->commit();
        } catch (\Exception $e) {            
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function insereRegraProporcionalizaEvento($nreventom, $nmeventoh){
        try{    
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            
            $regraDados = array('NREVENTOM'      => $nreventom, 
                                'NMEVENTOH'      => $nmeventoh, 
                                'DTMESCOMPETENC' => $dtmescompetenc, 
                                'NRORG'          => $nrorg,
                                'DTINIVIGENCIA'  => '01/01/2000', 
                                'DSOBSERVACAO'   => null, 
                                'IDDEBUG'        => $this->hcmParameters->getIdDebug(), 
                                'DTFIMVIGENCIA'  => null, 
                                'DSOBSERVACAO'   => null,
                                'DSREGRA'        => "VAL_EVEN(" . $nreventom . ")/30*(DIAS_NORM()+DIAS_ATEST())",
                                'NMREGRAH'       => "REGRA PROPORCIONALIZA EVENTO " . $nreventom);
                                
            if( empty($this->buscaRegra($regraDados)) ){
                $nrregram = $this->setRegra($regraDados);
                if(!empty($nrregram)){
                    $parametros = array('NREVENTOM' => $nreventom, 'NMEVENTOH' => $nmeventoh, 'DTMESCOMPETENC' => $dtmescompetenc, 'NRORG' => $nrorg);
            
                    $this->pontoRegraService->inserePontoRegraEventoProventoEventual([$nrregram], $parametros);
                }
            }
        } catch(\Exception $e){
            throw $e;
        }
        
    }
    
    public function insideParaPensãoAlimenEmFerias($nreventom, $nmeventoh){
        try{    
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            
            $regraDados = array('NREVENTOM'      => $nreventom, 
                                'NMEVENTOH'      => $nmeventoh, 
                                'DTMESCOMPETENC' => $dtmescompetenc, 
                                'NRORG'          => $nrorg,
                                'DTINIVIGENCIA'  => '01/01/2000', 
                                'DSOBSERVACAO'   => null, 
                                'IDDEBUG'        => $this->hcmParameters->getIdDebug(), 
                                'DTFIMVIGENCIA'  => null, 
                                'DSOBSERVACAO'   => null,
                                'DSREGRA'        => "(VAL_EVEN(3100)-VAL_EVEN(1140)-VAL_EVEN(1145))*REF_EVEN($nreventom)/100",
                                'NMREGRAH'       => "REGRA PARA PENSAO ALIMENTICIA EM FERIAS " . $nreventom);
                                
            if( empty($this->buscaRegra($regraDados)) ){
                $nrregram = $this->setRegra($regraDados);
                if(!empty($nrregram)){
                    $parametros = array('NREVENTOM' => $nreventom, 'NMEVENTOH' => $nmeventoh, 'DTMESCOMPETENC' => $dtmescompetenc, 'NRORG' => $nrorg);
            
                    $this->pontoRegraService->inserePontoRegraPensãoAlimenEmFerias([$nrregram], $parametros);
                }
            }
        } catch(\Exception $e){
            throw $e;
        }
        
    }
    
    public function insereRegrasEventoProventoEventual($nreventom, $nmeventoh, $idunidrefereven, $vrpercentualevento, $idtipoevento, $idtipoocorreven){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        $parametros = array('NREVENTOM' => $nreventom, 'NMEVENTOH' => $nmeventoh, 'DTMESCOMPETENC' => $dtmescompetenc, 'NRORG' => $nrorg);
        
        $this->gerenciaInclusaoRegrasEventoProventoEventual($idunidrefereven, $vrpercentualevento, $parametros);
    }
    
    public function gerenciaInclusaoRegrasEventoProventoEventual($idunidrefereven, $vrpercentualevento, $parametros){
        //insere as regras automáticas que variam conforme os valores dos campos VRPERCENTUALEVENTO e IDUNIDREFEREVEN
        try {
            $regras = array();
            $regraDados = array('DTINIVIGENCIA' => $this->hcmParameters->getDataAtual(true), 'DSOBSERVACAO' => null, 'IDDEBUG' => $this->hcmParameters->getIdDebug(), 'DTFIMVIGENCIA' => null, 'DSOBSERVACAO' => null);
            $dtinivigencia = $this->hcmParameters->getDataAtual(true);
            $iddebug = $this->hcmParameters->getIdDebug();
            $dsobservacao = null;
            $dtfimvigencia = null;
            $nreventom = $parametros['NREVENTOM'];
            
            if( empty($vrpercentualevento) && $idunidrefereven == 'DIA' ){
                $this->verificaUnidadeReferencia($nreventom, $this->hcmParameters->getNrorgTrabalho(), $idunidrefereven);
                array_push($regras, array("DSREGRA" => "SE(REF_EVEN(" . $nreventom . ")>0, (SALARIO()/30*REF_EVEN(" . $nreventom . ")), VAL_EVEN(" . $nreventom . "))", "NMREGRAH" => "REGRA DIA EVENTO " . $nreventom));
            } else if( empty($vrpercentualevento) && $idunidrefereven == 'HORA' ){
                $this->verificaUnidadeReferencia($nreventom, $this->hcmParameters->getNrorgTrabalho(), $idunidrefereven);
                array_push($regras, array("DSREGRA" => "SE(REF_EVEN(" . $nreventom . ")>0, (SALARIO()/(JORNADA_DIA()*30))*REF_EVEN(" . $nreventom . "), VAL_EVEN( " . $nreventom . "))", "NMREGRAH" => "REGRA HORA EVENTO " . $nreventom));
            } else if( !empty($vrpercentualevento) && $idunidrefereven == 'HORA' ){
                $this->verificaUnidadeReferencia($nreventom, $this->hcmParameters->getNrorgTrabalho(), 'HORA_PERCENT');
                array_push($regras, array("DSREGRA" => "SE(REF_EVEN(" . $nreventom . ")>0,SE(VAL_INCID(26)>0,VAL_INCID(26), SALARIO())/(JORNADA_DIA()*SE(DIAS_REM()>0, DIAS_REM(), 30))*REF_EVEN(" . $nreventom . ")*(" . $vrpercentualevento . "/100),VAL_EVEN(" . $nreventom . "))", "NMREGRAH" => "REGRA PERCENTUAL HORA EVENTO " . $nreventom));
            } else if( empty($vrpercentualevento) && $idunidrefereven == 'HORA_MINUTO' ){
                $this->verificaUnidadeReferencia($nreventom, $this->hcmParameters->getNrorgTrabalho(), $idunidrefereven);
                array_push($regras, array("DSREGRA" => "SE(REF_EVEN( " . $nreventom . ")>0, (SALARIO()/(JORNADA_DIA()*30))*CONV_MIN_DEC(REF_EVEN(" . $nreventom . ")), VAL_EVEN(" . $nreventom . "))", "NMREGRAH" => "REGRA HORA MINUTO EVENTO " . $nreventom));
            } else if( !empty($vrpercentualevento) && $idunidrefereven == 'HORA_MINUTO' ){
                $this->verificaUnidadeReferencia($nreventom, $this->hcmParameters->getNrorgTrabalho(), 'HORA_MINUTO_PERCENT');
                array_push($regras, array("DSREGRA" => "SE(REF_EVEN( " . $nreventom . ")>0,SE(VAL_INCID(26)>0,VAL_INCID(26), SALARIO())/(JORNADA_DIARIA*SE(DIAS_REM()>0, DIAS_REM(), 30))*CONV_MIN_DEC(REF_EVEN(" . $nreventom . "))*(" . $vrpercentualevento . "/100) ,VAL_EVEN(" . $nreventom . "))", "NMREGRAH" => "REGRA PERCENTUAL HORA MINUTO EVENTO " . $nreventom));
            } else {
                $this->verificaUnidadeReferencia($nreventom, $this->hcmParameters->getNrorgTrabalho(), '');
            }
            
            $regraDados = array_merge($regraDados, $parametros);
            
            if(!empty($regras)){
                foreach($regras as $key => $regra){
                    $regras[$key] = array_merge($regra, $regraDados);
                }
                $nrregram = $this->salvaRegra($regras, $parametros);
            }
            
            
            if(!empty($nrregram)){
                $this->pontoRegraService->inserePontoRegraEventoProventoEventual($nrregram, $parametros);
            }
        } catch(\Exception $e){
            throw $e;
        }
        
    }
    
    public function salvaRegra($regras, $parametros){
        try{
            $nrregrams = array();
            foreach($regras as $key => $regra){
              if( !empty($this->buscaRegra($regra)) ){
                  continue;
              }
              $nrregram = $this->setRegra($regra);
              array_push($nrregrams, $nrregram);
            }
            return $nrregrams;
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function setRegra($regra){
        
        try{
            $dataInclusao = DateUtil::getDataAtual();
            $cdOperadorInclusao = $this->hcmParameters->getCdoperador();
            $dtMesCompetenc = DateUtil::getDatadeString($regra['DTMESCOMPETENC'],DateUtil::FORMATO_BRASILEIRO, true);
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            
            $nrregram = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_REGRAM', 1, 12, true);
            $regram = new FpaRegram();
            $regrah = new FpaRegrah();
            
            $this->entityManager->beginTransaction();
            
            //regram
            $regram->setCdoperinclusao(array_key_exists('CDOPERADORINCLUSAO', $regra) ? $regra['CDOPERADORINCLUSAO'] : $cdOperadorInclusao);
            $regram->setCdoperultatu(array_key_exists('CDOPERULTATU', $regra) ? $regra['CDOPERULTATU'] : array_key_exists('CDOPERADORINCLUSAO', $regra) ? $regra['CDOPERADORINCLUSAO'] : $cdOperadorInclusao);
            $regram->setDtfimvigencia(array_key_exists('DTFIMVIGENCIA', $regra) ? DateUtil::getDatadeString($regra['DTFIMVIGENCIA'],DateUtil::FORMATO_BRASILEIRO, true) : null);
            $regram->setDtinclusao(array_key_exists('DTINCLUSAO', $regra) ? $regra['DTINCLUSAO'] : $dataInclusao);
            $regram->setDtinivigencia(DateUtil::getDatadeString($regra['DTINIVIGENCIA'],DateUtil::FORMATO_BRASILEIRO, true));
            $regram->setDtultatu(array_key_exists('DTULTATU', $regra) ? $regra['DTULTATU'] : (array_key_exists('DTINCLUSAO', $regra) ? $regra['DTINCLUSAO'] : $dataInclusao));
            $regram->setIddebug(array_key_exists('IDDEBUG', $regra) ? $regra['IDDEBUG'] : null);
            $regram->setNrorg(array_key_exists('NRORG', $regram) ? $regra['NRORG'] : $nrorg);
            $regram->setNrorginclusao(array_key_exists('NRORGINCLUSAO', $regra) ? $regra['NRORGINCLUSAO'] : (array_key_exists('NRORG', $regra) ? $regra['NRORG'] : $nrorg));
            $regram->setNrorgultatu(array_key_exists('NRORGULTATU', $regra) ? $regra['NRORGULTATU'] : (array_key_exists('NRORG', $regra) ? $regra['NRORG'] : $nrorg));
            $regram->setNrregram(array_key_exists('NRREGRAM', $regra) ? $regra['NRREGRAM'] : $nrregram);
            
            $this->entityManager->persist($regram);
            $this->entityManager->flush();
            
            //regrah
            $regrah->setCdoperinclusao(array_key_exists('CDOPERADORINCLUSAO', $regra) ? $regra['CDOPERADORINCLUSAO'] : $cdOperadorInclusao);
            $regrah->setCdoperultatu(array_key_exists('CDOPERULTATU', $regra) ? $regra['CDOPERULTATU'] : (array_key_exists('CDOPERADORINCLUSAO', $regra) ? $regra['CDOPERADORINCLUSAO'] : $cdOperadorInclusao));
            $regrah->setDsobservacao(array_key_exists('DSOBSERVACAO', $regra) ? $regra['DSOBSERVACAO'] : null);
            $regrah->setDsregra(array_key_exists('DSREGRA', $regra) ? $regra['DSREGRA'] : null);
            $regrah->setDtinclusao(array_key_exists('DTINCLUSAO', $regra) ? $regra['DTINCLUSAO'] : $dataInclusao);
            $regrah->setDtmescompetenc($dtMesCompetenc);
            $regrah->setDtultatu(array_key_exists('DTULTATU', $regra) ? $regra['DTULTATU'] : (array_key_exists('DTINCLUSAO', $regra) ? $regra['DTINCLUSAO'] : $dataInclusao));
            $regrah->setNmregrah($regra['NMREGRAH']);
            $regrah->setNreventom(array_key_exists('NREVENTOM', $regra) ? $regra['NREVENTOM'] : null);
            $regrah->setNrorg(array_key_exists('NRORG', $regram) ? $regra['NRORG'] : $nrorg);
            $regrah->setNrorginclusao(array_key_exists('NRORGINCLUSAO', $regra) ? $regra['NRORGINCLUSAO'] : (array_key_exists('NRORG', $regra) ? $regra['NRORG'] : $nrorg));
            $regrah->setNrorgultatu(array_key_exists('NRORGULTATU', $regra) ? $regra['NRORGULTATU'] : (array_key_exists('NRORG', $regra) ? $regra['NRORG'] : $nrorg));
            $regrah->setNrregrah(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_REGRAH', 1, 12, true));
            $regrah->setNrregram(array_key_exists('NRREGRAM', $regra) ? $regra['NRREGRAM'] : $nrregram);
            
            $this->entityManager->persist($regrah);
            $this->entityManager->flush();
            $this->entityManager->commit();
            
            return $nrregram;
        } catch(\Exception $e){
                $this->entityManager->rollback();
                throw $e;
            }
        
    }
    
    public function buscaRegra($regra) {
        return $this->entityManager->getRepository(Repositories::FPA_REGRAH)->findOneBy(array( "nrorg" => $regra['NRORG'], "nreventom" => $regra['NREVENTOM'], "dsregra" => $regra['DSREGRA'] ));
    }
    
    public function verificaUnidadeReferencia($nreventom, $nrorg, $referencia) {
        
        $nmregras = array(array("NMREGRAH" => "REGRA DIA EVENTO " . $nreventom),
                          array("NMREGRAH" => "REGRA HORA EVENTO " . $nreventom),
                          array("NMREGRAH" => "REGRA PERCENTUAL HORA EVENTO " . $nreventom),
                          array("NMREGRAH" => "REGRA HORA MINUTO EVENTO " . $nreventom),
                          array("NMREGRAH" => "REGRA PERCENTUAL HORA MINUTO EVENTO " . $nreventom));
                          
        switch ($referencia) {
            case 'DIA':
                unset($nmregras[0]);
                break;
            case 'HORA':
                unset($nmregras[1]);
                break;
            case 'HORA_PERCENT':
                unset($nmregras[2]);
                break;
            case 'HORA_MINUTO':
                unset($nmregras[3]);
                break;
            case 'HORA_MINUTO_PERCENT':
                unset($nmregras[4]);
                break;
        }

        foreach($nmregras as $nmregra){
            $regra = $this->entityManager->getRepository(Repositories::FPA_REGRAH)->findOneBy(array( "nrorg" => $nrorg, "nreventom" => $nreventom, "nmregrah" => $nmregra));
            
            if(!empty($regra) ){
                $dataset = array("NRREGRAM" => $regra->getNrregram(), "NRREGRAH" => $regra->getNrregrah(), "NRORG" => $nrorg);
                $this->pontoRegraService->deletePontoRegra($dataset);
                $this->deleteRegra($dataset);
            }
        }
    }
    
    
    public function deleteRegraAlim($nreventom, $nrorg){
        
        $regra = $this->entityManager->getRepository(Repositories::FPA_REGRAH)->findOneBy(array( "nrorg" => $nrorg, "nreventom" => $nreventom, "nmregrah" => "REGRA PARA PENSAO ALIMENTICIA EM FERIAS " . $nreventom));
        
        if(!empty($regra) ){
            $dataset = array("NRREGRAM" => $regra->getNrregram(), "NRREGRAH" => $regra->getNrregrah(), "NRORG" => $nrorg);
            $this->pontoRegraService->deletePontoRegra($dataset);
            $this->deleteRegra($dataset);
        }
    }
    
    public function deleteRegraPropor($nreventom, $nrorg){
        
        $regra = $this->entityManager->getRepository(Repositories::FPA_REGRAH)->findOneBy(array( "nrorg" => $nrorg, "nreventom" => $nreventom, "nmregrah" => "REGRA PROPORCIONALIZA EVENTO " . $nreventom));
        
        if(!empty($regra) ){
            $dataset = array("NRREGRAM" => $regra->getNrregram(), "NRREGRAH" => $regra->getNrregrah(), "NRORG" => $nrorg);
            $this->pontoRegraService->deletePontoRegra($dataset);
            $this->deleteRegra($dataset);
        }
    }
}