<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;

class ParametroEstruturaController extends Crud {

    protected $dataSourceName_get = 'gpe_paramestrut_get';
    protected $ParametroEstruturaService;
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', "P_DTMESCOMPETENC", "P_NRTIPOESTRUTURA", "P_EXIBEREGISTRO");
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\Geral\Services\ParametroEstruturaService $ParametroEstruturaService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->parametroEstruturaService = $ParametroEstruturaService;
        parent::__construct($dataSourceManager);
    }

    public function getParametroEstrutura(DTO\Request\Filter $request, DTO\Response $response) {   
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function deleteParametroEstrutura(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->parametroEstruturaService->deleteParametroEstrutura($dataset);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function verificaParamEstruturaDuplic(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $duplicidade = $this->parametroEstruturaService->verificaParamEstruturaDuplic($dataset);
            
            if($duplicidade == true){
                $response->addNotification(new Notification("Jâ existe um parametro para a estrutura escolhida", Notification::TYPE_ERROR));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
    public function replicaParametroFolha(DTO\Request\DataSet $request, DTO\Response $response) {
        $row=$request->getDataSet()->getRows();
        $row=$row[0];
        $nmestruturam="";
        try {
            if (is_null($row['NRORG'])) {
                $response->addNotification(new Notification("A estrutura posicionada não possui parâmetro para replicação",Notification::TYPE_ERROR));
            } else {
                foreach ($row["NRESTRUTURAM_ALVO"]as $nrestruturam){
                    
                    if($this->parametroEstruturaService->replicaParametroFolha($row, $nrestruturam, $nmestruturam)){
                        $response->addNotification(new Notification("Parâmetro replicado com sucesso para a estrutura $nmestruturam", Notification::TYPE_SUCCESS));
                    }else{
                        $response->addNotification(new Notification("Parâmetro existente para a estrutura $nmestruturam",Notification::TYPE_ERROR));
                    }
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

} 