<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class ProcessoTrabController extends Crud {

    protected $dataSourceName = '';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_NRTPMODALIDCAL');
    protected $processoTrabService;

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters,\HCM\Geral\Services\ProcessoTrabService $processoTrabService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->processoTrabService = $processoTrabService;
        parent::__construct($dataSourceManager);
    }

    public function getProcessoTrab (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("gpe_processotrab_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
     public function setProcessoTrab(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "gpe_processotrab";
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRPROCESSTRAB', $row)) {
                    if ($row['NRPROCESSTRAB'] === '') {
                        $row['NRPROCESSTRAB'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_PROCESSTRAB', 1, 12, true); 
                    }
                } else {
                    $row['NRPROCESSTRAB'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_PROCESSTRAB', 1, 12, true); 
                }

            }
            
            $row['IDATIVO'] = isset($row['IDATIVO']) ? $row['IDATIVO'] : 'S';
            $row['NRPARCNEGOCIO'] = isset($row['NRPARCNEGOCIO']) ? $row['NRPARCNEGOCIO'] : null;
            $row['CDMUNIVARPRTRAB'] = isset($row['CDMUNIVARPRTRAB']) ? $row['CDMUNIVARPRTRAB'] : null;
            $row['CDVARAPROCETRAB'] = isset($row['CDVARAPROCETRAB']) ? $row['CDVARAPROCETRAB'] : null;
            $row['DSOBSERVACAO'] = isset($row['DSOBSERVACAO']) ? $row['DSOBSERVACAO'] : null;
            
            parent::save($request, $response);
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteProcessoTrab(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $msg = $this->processoTrabService->validadelete($request->getDataSet()->getRows()[0]);
            if(!empty($msg)) {
                $response->addDataSet(new DataSet("msg", $msg));
                return;
            }
            $this->dataSourceName = "gpe_processotrab";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    public function getInfoProcTrab (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("gpe_infoproctrab_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
     public function setInfoProcTrab(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "gpe_infoproctrab";
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRINFOPROCTRAB', $row)) {
                    if ($row['NRINFOPROCTRAB'] === '') {
                        $row['NRINFOPROCTRAB'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_INFOPROCTRAB', 1, 12, true); 
                    }
                } else {
                    $row['NRINFOPROCTRAB'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_INFOPROCTRAB', 1, 12, true); 
                }
            }
            
            $row['IDATIVO'] = isset($row['IDATIVO'])?$row['IDATIVO']:'S';
            
            parent::save($request, $response);
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteInfoProcTrab(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "gpe_infoproctrab";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}