<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;
use HCM\Util\HCMParameters;

class HierarquiaGridRepository {
    
    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function retornaGridExistenteValido($nrtipoestrutsup, $nrestruturamrel, $nrorg, $nrHierarquiaGrid){
        
        $params = array('P_NRTIPOESTRUTSUP' => $nrtipoestrutsup,
                        'P_NRESTRUTURAMREL' => $nrestruturamrel,
                        'P_NRORG' => $nrorg,
                        'P_DTMESCOMPETENC' => $this->hcmParameters->getDtCompetencia(),
                        'P_NRHIERARQUIAGRID' => $nrHierarquiaGrid
                        );
                        
        $query = "SELECT HIERARQUIAORG.* 
                    FROM HIERARQUIAORG
                    JOIN TIPOHIERARQORG TH ON TH.NRTIPOHIERARQORG = HIERARQUIAORG.NRTIPOHIERARQORG
                                          AND TH.NRTIPOESTRUTSUP = :P_NRTIPOESTRUTSUP
                    WHERE HIERARQUIAORG.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
                    AND (HIERARQUIAORG.DTFIMVIGENCIA >= LAST_DAY(:P_DTMESCOMPETENC) OR HIERARQUIAORG.DTFIMVIGENCIA IS NULL)
                    AND HIERARQUIAORG.NRESTRUTURAMREL = :P_NRESTRUTURAMREL
                    AND HIERARQUIAORG.NRORG = :P_NRORG
                    AND (HIERARQUIAORG.NRHIERARQUIAORG <> :P_NRHIERARQUIAGRID OR :P_NRHIERARQUIAGRID IS NULL)";
                     
        return $this->connection->fetchAll($query, $params);
                     
    }
    
    public function retornaHierarquiaGridExistente($nrtipohierarqorg, $nrestruturamrel, $nrorg){
        
        $params = array('P_NRTIPOHIERARQORG' => $nrtipohierarqorg,
                        'P_NRESTRUTURAMREL' => $nrestruturamrel,
                        'P_NRORG' => $nrorg);
        $query = "SELECT HIERARQUIAORG.* 
                        FROM HIERARQUIAORG
                   WHERE HIERARQUIAORG.NRTIPOHIERARQORG = :P_NRTIPOHIERARQORG
                     AND HIERARQUIAORG.DTINIVIGENCIA = (SELECT MAX(HO.DTINIVIGENCIA) 
                                            FROM HIERARQUIAORG HO
                                           WHERE HO.NRTIPOHIERARQORG = HIERARQUIAORG.NRTIPOHIERARQORG
                                             AND HO.NRESTRUTURAMREL = HIERARQUIAORG.NRESTRUTURAMREL
                                             AND HO.NRORG = HIERARQUIAORG.NRORG)
                     AND HIERARQUIAORG.NRESTRUTURAMREL = :P_NRESTRUTURAMREL
                     AND HIERARQUIAORG.NRORG = :P_NRORG";
                     
        return $this->connection->fetchAll($query, $params);
                     
    }
    
    
    
    public function getHierarquiaGridTree($nrestruturamrel, $nrestruturamsup, $nrtipohierarqorg, $nrtipoestrutsup, $where_placeholder, $buttonFloatingSearch){
        
        if($nrestruturamsup) $nrestruturamsup = " AND HO.NRESTRUTURAMSUP = $nrestruturamsup ";
        
        if($nrestruturamrel) $nrestruturamrel = " AND HO.NRESTRUTURAMREL = $nrestruturamrel ";

        $query = "
        SELECT * FROM (
        
            SELECT
                HO.NRHIERARQUIAORG,
                HO.NRORG,
                HO.NRTIPOHIERARQORG,
                HO.NMHIERARQUIAORG,
                TO_CHAR(DECODE(HO.NRESTRUTURAMSUP, HO.NRESTRUTURAMREL, NULL, HO.NRESTRUTURAMSUP)) AS NRESTRUTURAMSUP,
                TO_CHAR(HO.NRESTRUTURAMREL) AS NRESTRUTURAMREL,
                HO.DTINIVIGENCIA,
                HO.DTFIMVIGENCIA,
                HO.DTINCLUSAO,
                HO.DTULTATU,
                HO.NRORGINCLUSAO,
                HO.CDOPERINCLUSAO,
                HO.NRORGULTATU,
                HO.CDOPERULTATU,
                HO.IDATIVO,
                EREL.NMESTRUTURAM AS NMESTRUTURAMREL,
                DECODE(HO.NRESTRUTURAMSUP, HO.NRESTRUTURAMREL, NULL, ESUP.NMESTRUTURAM ) NMESTRUTURAMSUP,
                TH.NMTIPOHIERARQORG,
                TH.NMTIPOHIERARQORG AS NMTIPOHIERARQORG_DESCRICAO
            FROM
                HIERARQUIAORG HO 
            INNER JOIN
                TIPOHIERARQORG TH 
                    ON HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG 
            INNER JOIN
                ESTRUTURAM EREL 
                    ON HO.NRESTRUTURAMREL = EREL.NRESTRUTURAM 
                    AND (
                        HO.NRORG = EREL.NRORG
                    ) 
            LEFT JOIN
                ESTRUTURAM ESUP 
                    ON HO.NRESTRUTURAMSUP = ESUP.NRESTRUTURAM 
                    AND (
                        HO.NRORG = ESUP.NRORG
                    ) 
            WHERE
                HO.NRORG = :P_NRORG
                AND HO.NRESTRUTURAMSUP <> HO.NRESTRUTURAMREL
                AND HO.NRTIPOHIERARQORG = :P_NRTIPOHIERARQORG
                $nrestruturamsup
                $nrestruturamrel
                AND HO.DTINIVIGENCIA    = ( SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG
                                     WHERE DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC) 
                                       AND NRORG            = HO.NRORG
                                       AND NRESTRUTURAMREL  = HO.NRESTRUTURAMREL
                                       AND NRTIPOHIERARQORG = HO.NRTIPOHIERARQORG )
        UNION 
        
            SELECT DISTINCT
                NULL AS NRHIERARQUIAORG,
                NULL AS NRORG,
                HO.NRTIPOHIERARQORG,
                TE.NMTIPOESTRUTURA AS NMHIERARQUIAORG,
                NULL AS NRESTRUTURAMSUP,
                TO_CHAR(HO.NRESTRUTURAMSUP) AS NRESTRUTURAMREL,
                NULL AS DTINIVIGENCIA,
                NULL AS DTFIMVIGENCIA,
                NULL AS DTINCLUSAO,
                NULL AS DTULTATU,
                NULL AS NRORGINCLUSAO,
                NULL AS CDOPERINCLUSAO,
                NULL AS NRORGULTATU,
                NULL AS CDOPERULTATU,
                NULL AS IDATIVO,
                ESUP.NMESTRUTURAM AS NMESTRUTURAMREL,
                NULL AS NMESTRUTURAMSUP,
                TE.NMTIPOESTRUTURA AS NMTIPOHIERARQORG,
                TE.NMTIPOESTRUTURA AS NMTIPOHIERARQORG_DESCRICAO
        
            FROM
                HIERARQUIAORG HO 
            INNER JOIN
                TIPOHIERARQORG TH 
                    ON HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG 
            INNER JOIN
                ESTRUTURAM ESUP 
                    ON HO.NRESTRUTURAMSUP = ESUP.NRESTRUTURAM 
                    AND (
                        HO.NRORG = ESUP.NRORG
                    )
            INNER JOIN TIPOESTRUTURA TE 
                ON TE.NRTIPOESTRUTURA = ESUP.NRTIPOESTRUTURA
                    
            WHERE
                HO.NRORG = :P_NRORG
                AND (ESUP.NRTIPOESTRUTURA = :P_NRTIPOESTRUTSUP OR :P_NRTIPOESTRUTSUP IS NULL)
                $nrestruturamsup
                $nrestruturamrel
                AND HO.NRTIPOHIERARQORG = :P_NRTIPOHIERARQORG
                AND HO.DTINIVIGENCIA    = ( SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG
                                     WHERE DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC) 
                                       AND NRORG            = HO.NRORG
                                       AND NRESTRUTURAMREL  = HO.NRESTRUTURAMREL
                                       AND NRTIPOHIERARQORG = HO.NRTIPOHIERARQORG )            

                                          
        ) WHERE $where_placeholder $buttonFloatingSearch                               
        ";
        $params = [
                    "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                    "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                    "P_NRTIPOESTRUTSUP" => $nrtipoestrutsup,
                    "P_NRTIPOHIERARQORG" => $nrtipohierarqorg,
                  ];
                  
        return $this->connection->fetchAll($query, $params);                  

    }

    public function getHierarquiaGrid($nrestruturamrel, $nrestruturamsup, $where_placeholder, $buttonFloatingSearch){
        
        if($nrestruturamsup) $nrestruturamsup = " AND HO.NRESTRUTURAMSUP = $nrestruturamsup ";
        
        if($nrestruturamrel) $nrestruturamrel = " AND HO.NRESTRUTURAMREL = $nrestruturamrel ";

        $query = "
        SELECT * FROM (
        
            SELECT
                HO.NRHIERARQUIAORG,
                HO.NRORG,
                HO.NRTIPOHIERARQORG,
                HO.NMHIERARQUIAORG,
                TO_CHAR(DECODE(
                    HO.NRESTRUTURAMSUP,
                    HO.NRESTRUTURAMREL,
                    NULL,
                    HO.NRESTRUTURAMSUP
                )) AS NRESTRUTURAMSUP,
                TO_CHAR(HO.NRESTRUTURAMREL) AS NRESTRUTURAMREL,
                HO.DTINIVIGENCIA,
                HO.DTFIMVIGENCIA,
                HO.DTINCLUSAO,
                HO.DTULTATU,
                HO.NRORGINCLUSAO,
                HO.CDOPERINCLUSAO,
                HO.NRORGULTATU,
                HO.CDOPERULTATU,
                HO.IDATIVO,
                EREL.NMESTRUTURAM AS NMESTRUTURAMREL,
                ESUP.NRTIPOESTRUTURA,
                DECODE(
                    HO.NRESTRUTURAMSUP,
                    HO.NRESTRUTURAMREL,
                    NULL,
                    ESUP.NMESTRUTURAM
                ) NMESTRUTURAMSUP,
                TH.NMTIPOHIERARQORG,
                NVL(EQV.QNT_VINCULOS, 0) NUMERODEVINC
            FROM
                HIERARQUIAORG HO 
            LEFT JOIN
                TIPOHIERARQORG TH 
                    ON HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG 
            LEFT JOIN
                ESTRUTURAM EREL 
                    ON HO.NRESTRUTURAMREL = EREL.NRESTRUTURAM 
                    AND (
                        HO.NRORG = EREL.NRORG 
                    ) 
            LEFT JOIN
                ESTRUTURAM ESUP 
                    ON HO.NRESTRUTURAMSUP = ESUP.NRESTRUTURAM 
                    AND (
                        HO.NRORG = ESUP.NRORG 
                    )
            LEFT JOIN (
                SELECT NRESTRUTURAM, COUNT(*) QNT_VINCULOS
                FROM (
                    SELECT
                        NRESTRUTLEGAL,
                        NRESTRUTGEREN
                    FROM GPE_VINCULOM VM
                    JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                        AND VH.NRVINCULOM = VM.NRVINCULOM
                        AND VH.DTMESCOMPETENC = (
                            SELECT MAX(DTMESCOMPETENC)
                            FROM GPE_VINCULOH
                            WHERE NRORG = VH.NRORG
                                AND NRVINCULOM = VH.NRVINCULOM
                                AND DTMESCOMPETENC <= :P_DTMESCOMPETENC
                            GROUP BY NRVINCULOM
                        )
                    WHERE VM.NRORG = :P_NRORG
                        AND ( VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC
                            OR VM.DTRESCISAOVINC IS NULL )
                )
                UNPIVOT (
                    NRESTRUTURAM FOR TIPO_ESTRUTURA IN (
                        NRESTRUTLEGAL,
                        NRESTRUTGEREN
                    )
                )
                GROUP BY NRESTRUTURAM
            ) EQV ON EQV.NRESTRUTURAM = HO.NRESTRUTURAMREL
            WHERE
                HO.NRORG = :P_NRORG 
                $nrestruturamsup
                $nrestruturamrel
                AND ( HO.DTFIMVIGENCIA >= :P_DTMESCOMPETENC
                      OR HO.DTFIMVIGENCIA IS NULL )
                                          
        ) WHERE $where_placeholder $buttonFloatingSearch                               
        ";
        $params = [
                    "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                    "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                  ];
    
        return $this->connection->fetchAll($query, $params);                  

    }
    
    public function retornaSindicatosComHierarquia($nrestruturamsup){
        $query = "
        SELECT
            TPH.NRTIPOESTRUTSUP,
            TPH.NRTIPOESTRUTREL,
            TPH.NRORG,
            HO.NRESTRUTURAMREL,
            ESTRUT_SUP.NRESTRUTURAM    AS NRESTRUTLEGAL,
            ESTRUT_SUP.NMESTRUTURAM    AS NMRAZSOCESTRUT_LEGAL,
            ESTRUT_SUP.NRTIPOESTRUTURA AS NRTIPOESTRUTURA_LEGAL
        FROM
                 TIPOHIERARQORG TPH
            JOIN HIERARQUIAORG HO ON HO.NRTIPOHIERARQORG = TPH.NRTIPOHIERARQORG
                                     AND ( HO.NRORG = :P_NRORG
                                           OR HO.NRORG = :P_NRORG_PADRAO )
            JOIN GPE_PARAMORG PO ON TPH.NRTIPOESTRUTSUP = PO.NRTPESTRUTLEGAL 
                                    AND  (PO.NRORG = :P_NRORG
                                           OR PO.NRORG = :P_NRORG_PADRAO )
            JOIN ESTRUTURAM  ESTRUT_SUP ON ESTRUT_SUP.NRESTRUTURAM = HO.NRESTRUTURAMSUP
                                          AND ESTRUT_SUP.NRORG = :P_NRORG
                                          AND ESTRUT_SUP.NRTIPOESTRUTURA = PO.NRTPESTRUTLEGAL
        WHERE
            (TPH.NRORG = :NRORG OR TPH.NRORG = :P_NRORG_PADRAO) AND
            TPH.NRTIPOESTRUTREL = 10 AND
            ESTRUT_SUP.NRESTRUTURAM = :P_NRESTRUTLEGAL;
        ";
        
        $params = [
                    "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                    "P_NRORG_PADRAO" => $this->hcmParameters->getNrorgPadrao(),
                    "P_NRESTRUTLEGAL" => $nrestruturamsup,
                  ];
    
        return $this->connection->fetchAll($query, $params);           
        
    }
}