<?php
namespace Util;

use Util\Repositories;
use Teknisa\ZeedhiBundle\Remote\FileUploader\ApiKeyProvider;
use Zeedhi\Framework\Session\Session;

class HCMAPIKeyProvider implements ApiKeyProvider{

    const FILE_UPLOADER_API_KEY = "fileUploaderApiKey";

    /** @var Session */
    protected $session;

    /**
     * SessionAPIKeyProvider constructor.
     * @param Session $session
     */
    public function __construct(Session $session, array $apiKeyMapping) {
        $this->session = $session;
        $this->apiKeyMapping = $apiKeyMapping;
        
        $this->entityManager = \Setforms2\Shared\Registry::getEntityManager();;
    }

    /**
     * @param string $key Store the api key in session to avoid constant access to OM remote server.
     */
    public function setFileUploaderApiKey($key) {
        $this->session->set(self::FILE_UPLOADER_API_KEY, $key);
    }

    /**
     * {@inheritdoc}
     * Get apiKey from fileServer.xml static file, or if not found, from DB organization table
     */
    public function getFileUploaderApiKey() {
        $key = null;
        
        $nrorg = $this->session->get('NRORG');
        
        if(isset($this->apiKeyMapping[$nrorg])) {
            $key =  $this->apiKeyMapping[$nrorg];
        } else {
            $organizacao = $this->entityManager->getRepository(Repositories::ORGANIZACAO)->findOneBy(array('nrorg' => $nrorg));
            if (is_object($organizacao)) {
                $key = $organizacao->getDsfsapikey();
            }
        }

        return $key;
    }
}